import java.applet.*;
import java.util.*;
import java.io.*;
import java.awt.Point;
import java.awt.*;
import javax.swing.tree.*;

/**
 * Die zentrale Klasse des Modells.
**/
public class Agent implements Serializable, MutableTreeNode{
	/** die Anzahl der (vom Benutzer) geschaffenen Agenten */
	public static int agentNum = 0;
	
	/** der aktuelle Nahrungsvorrat */
	private float food;
	/** das Alter (in Runden) */
	private int age;
	/** die Runde, in der der Agent geboren wurde */
	private int birthDay;
	/** die Runde, in der der Agent gestorben ist, -1 wenn er noch lebt */
	private int deathDay;	
	/** der eindeutige name, mit Hinweisen auf die Vorfahren */
	private String name;
	/** die aktuelle X-Koordinate */
	private int x = -1;
	/** die aktuelle Y-Koordinate*/
	private int y = -1;
	/** das Gedchtnis */
	private Memory memory;
	/** das Fenster zu Anzeige des Gedchtnisses, null falls keines geffnet */
	private MemoryMonitor memoryMonitor = null;
	/** das Fenster zur Anzeige von Informationen zum Agenten, null falls keines geffnet */
	private AgentMonitor agentMonitor = null;	
	/** die Anzahl an direkten Nachkommen */
	private int numOfChildren=0;
	/** auf welches Feld geht der nchste Schritt */
	private Point whereToMove;
	/** die Satisfaction des Agenten (siehe dort) */
	private Satisfaction sat;
	/** womit war der Agent gerade beschftigt */
	private String lastAction = "nothing";
	/** die Liste der direkten Nachkommen */
	private Vector children;
	/** der Eltern-Agent */
	private Agent parent;
	/** ist der Agent zur Zeit Koordinator */
	private boolean isCoordinator = false;
	/** der Koordinator des Agenten, oder null, falls der Agent nicht koordiniert wird */
	private Agent coordinator = null;
	/** das vom Koordinator ermittelte Zielfeld, null falls keines oder falls es schon erreicht ist */
	private Point target = null;
	/** eine Liste aller Agenten im Blickfeld */
	private Vector agentsInRange;
	/** der Koordinator mit der bisher besten Bewertung (der immernoch koordiniert), null falls keiner */
	private Agent bestCoordinator = null;
	/** die Bewertung dieses Koordinators */
	private float bestCoordinatorValue = 0f;
	/** gibt es einen Koordinator in Reichweite */
	private boolean coordinatorInRange;
	/** die Liste von allen dem Agenten bekannten noch aktiven Koordinatoren */
	private Vector knownCoordinators = new Vector();
	/** die Bewertungen derselben */
	private Vector knownCoordinatorsValues = new Vector();
	/** die Liste der Untergebenen (falls der Agent koordiniert) */
	private Vector subordinates = new Vector();
	/** die Liste der Zielfelder der Untergebenen */
	private Vector targets = new Vector();
	/** kann sich der Agent in dieser Runde bewegen */
	public boolean canMove;
	/** kann der Agent in dieser Runde Nahrung sammeln */
	public boolean canGather;
	/** kann sich der Agent in dieser Runde vermehren */
	public boolean canBreed;
	/** kann sich der Agent in dieser Runde einem Koordinator unterordnen */
	public boolean canSubordinate;
	/** kann der Agent in dieser Runde eine Unterordnung beenden */
	public boolean canUnsubordinate;
	/** kann der Agent in dieser Runde Koordinator werden */
	public boolean canStartCoordinating;
	/** kann der Agent in dieser Runde sein Koordinatorendasein beenden */
	public boolean canEndCoordinating;
	/** in dieser Runde wurde der Agent Koordinator */
	public int startCoordinatingDate = -1;
	/** in dieser Runde ordnete sich der Agent dem aktuellen Koordinator unter */
	public int subordinationDate = -1;
	/** mit diesem Nahrungsvorrat startete der Agent seine aktuelle Unterordnung */
	private float subordinationStartValue;

	/** soviel Nahrung verbraucht der Agent jede Runde */
	private float eatFoodPR;
	/** in diesem Alter wird der Agent fruchtbar */
	private float pubertyAge;
	/** die Maximalmenge der pro Runde gesammelten Nahrung */
	private float gatherAmount;
	/** soviel Nahrung belsst der Agent mindestens auf dem Feld */
	private float gatherRest;
	/** soviel Nahrung gibt er bei jeder Vermehrung an seine Nachkommen weiter */
	private float childFood;
	/** nur solange kann er Informationen ber Felder in seinem Gedchtnis behalten */
	private float memoryMaxAge;
	/** Felder bis zu diesem Abstand kann er wahrnehmen */
	private float viewingRange;
	/** Informationen ber Felder bis zu diesem Abstand bezieht der Agent in seine Planungen mit ein */
	private float planningRange;
	/** Gewichtung des Bedrfnisses zu berleben */
	private float survivalWeight;
	/** Gewichtung des Bedrfnisses nach Reichtum */
	private float wealthWeight;
	/** Gewichtung des Bedrfnisses Neugier */
	private float curiosityWeight;
	/** Gewichtung des Bedrfnisses sich fortzupflanzen */
	private float breedingWeight;
	/** Gewichtung des Bedrfnisses Einfluss (auf Untergebene) auszuben */
	private float influenceWeight;

	
	/**
	 * Erschafft einen neuen Agenten. Danach mssen noch diverse Werte gesetzt werden.
	 * Mit mutateAndSetParameters, setParameters oder setDefaultParameters.
	 * @param parent der Eltern-Agent.
	**/
	public Agent(Agent parent){
		age = 0;
		memory = new Memory(this);
		birthDay = World.round;
		deathDay = -1;
		sat = new Satisfaction(this);
		children = new Vector();
		this.parent = parent;
		if (parent != null){
			this.parent.insert(this, this.parent.getChildCount());
		}
	}
	
	/**
	 * Liefert die Anzahl der bisher geschaffenen (nicht der geborenen) Agenten. Muss nicht mit der Zahl der
	 * lebenden existierenden Agenten bereinstimmen.
	 * @return Die Zahl der (vom Benutzer) geschaffenen Agenten als int.
	**/
	public static int getNum(){
		return(agentNum);
	}
	
	/**
	 * Der Agent ist dran. ber diese Methode wird der Agent von Scheduler angesprochen.
	**/
	public void go(){	
		int maxChilds = World.getInt("Agent.maxNumOfChildren");
		age++;
		if (isCoordinator){
			food = food - eatFoodPR * World.getFloat("Agent.coordinatorsFoodPRFactor");
		}
		else{
			food = food - eatFoodPR;
		}
		if (World.getBoolean("Agent.lifeExpectancy")){
			float rnd = World.random.nextFloat();
			float leMin = World.getFloat("Agent.lifeExpectancyMin");
			float leMax = World.getFloat("Agent.lifeExpectancyMax");
			rnd = rnd * (leMax - leMin) + leMin;
			
			if (age > rnd){
				deathDay = World.round;
				//denSubordinates Bescheid sagen
				for (int i=0; i<subordinates.size(); i++){
					((Agent)subordinates.elementAt(i)).coordinationTerminated();
				}
			}
		}
		if (food > 0 && deathDay == -1){	
			lookAround((int)viewingRange);
			if (memoryMonitor != null){
				memoryMonitor.update();
			}
			
			// was kann der Agent aufgrund der aktuellen Situation berhaupt tun:
			Vector vTemp = World.getCoordinatesInDistance(x, y, 1);
			canMove = false;
			for (int i=0; i<vTemp.size();i++){
				Point p = (Point)vTemp.elementAt(i);
				if (!World.fields[p.x][p.y].hasLivingAgent()){
					canMove = true;
				}
			}
			canBreed = (World.getFreeField(x, y) != null) && food > childFood + eatFoodPR;
			if (maxChilds >= 0){
				canBreed = canBreed && (numOfChildren < maxChilds);
			}
			if (World.getBoolean("Agent.breedRestrictedToNeighbourhood")){
				canBreed = canBreed && canMove; //freies Nachbarfeld
			}
			// falls noch nicht subordinated, nicht selber Koordinator und Koordinator in "Reichweite"
			canGather = World.getField(x, y).getFood() > gatherRest;
			canSubordinate = coordinator == null && !isCoordinator && coordinatorInRange; //ToDo mehrstufige Hierarchien
			canUnsubordinate = coordinator != null; //Ausstieg aus der Koordination jederzeit mglich
			canStartCoordinating = !isCoordinator && coordinator == null; //ToDo mehrstufige Hierarchien
			canEndCoordinating = isCoordinator;
			if (World.getBoolean("Agent.enableHierarchy") == false){
				canSubordinate = false;
				canUnsubordinate = false;
				canStartCoordinating = false;
				canEndCoordinating = false;
			}
			
			//falls koordiniert, Koordinator kontakten
			if (coordinator != null){
				try{
					target = coordinator.getTarget(this, (int)(food / eatFoodPR));
				}
				catch(Exception e){
					System.out.println("in Agent.go: " + e);
					System.out.println("this: " + name);
				}
			}
			
			int action = decide();
			if (action == Satisfaction.MOVE){	//
				move(whereToMove);
				lastAction = "move";
				lookAround((int)viewingRange);
			}
			else if (action == Satisfaction.GATHER){
				gather();
				lastAction = "gather";
			}
			else if (action == Satisfaction.BREED){
				breed();
				lastAction = "breed";
				lookAround((int)viewingRange);
			}
			else if (action == Satisfaction.SUBORDINATE){
				subordinate();
				lastAction = "subordinate";
			}
			else if (action == Satisfaction.UNSUBORDINATE){
				unsubordinate();
				lastAction = "unsubordinate";
			}
			else if (action == Satisfaction.START_COORDINATING){
				startCoordinating();
				lastAction = "startCoordinating";
			}
			else if (action == Satisfaction.END_COORDINATING){
				endCoordinating();
				lastAction = "endCoordinating";
			}
			else if (action == Satisfaction.REST){
				rest();
				lastAction = "rest";
			}
			else{
				//h??
				System.out.println("Probleme in Agent.go");
			}
		}
		else{ //food <= 0
			//food = 0f;
			if (deathDay == -1){
				deathDay = World.round;
				if (World.sound) World.deathSound.play();
				//denSubordinates Bescheid sagen
				for (int i=0; i<subordinates.size(); i++){
					((Agent)subordinates.elementAt(i)).coordinatorDied();
				}
			}
		}
		if ((age < pubertyAge) || (numOfChildren >= maxChilds) || (lastAction.equals("breed"))){
			sat.setSatis(Satisfaction.BREEDING, 1f);
		}
		else{
			sat.setSatis(Satisfaction.BREEDING, sat.getSatis(Satisfaction.BREEDING) 
																	* World.getFloat("Agent.birthRecreationFactor"));
		}
		if (agentMonitor != null){
			agentMonitor.update();
		}
		if (memoryMonitor != null){
			memoryMonitor.update();
		}
	}
	
	/**
	 * Nimmt die nhere Umgebung des Agenten wahr.
	 * D.h. die Information in memory wird fr alle Felder in Sichtweite aktualisiert.
	 * Zustzlich werden noch die Informationen ber mgliche Koordinatoren aktualisiert.
	 * @param range Sichtweite in alle acht Richtungen.
	**/
	public void lookAround(int range){
		int i;
		Field f;
		Agent a;
		Point p;
		Vector v;
		String name;
		
		// knownCoordinators und bestCoordinator updaten
		bestCoordinator = null;
		bestCoordinatorValue = Float.NEGATIVE_INFINITY;
		for (i=0; i<knownCoordinators.size(); i++){
			a = (Agent)knownCoordinators.elementAt(i);
			if (!a.isCoordinator() || !a.isAlive()){
				knownCoordinators.remove(i);
				knownCoordinatorsValues.remove(i);
				i--;
			}
			else{
				if (((Float)knownCoordinatorsValues.elementAt(i)).floatValue() > bestCoordinatorValue){
					bestCoordinatorValue = ((Float)knownCoordinatorsValues.elementAt(i)).floatValue();
					bestCoordinator = (Agent)knownCoordinators.elementAt(i);
				}
			}
		}
		agentsInRange = new Vector();
		coordinatorInRange = (knownCoordinators.size() > 0);
		// Schleife fr die einzelnen Reichweiten
		for (i=0; i<=range; i++){
			v = World.getRandomizedCoordinatesInDistance(x, y, i);
			while (v.size() > 0){
				p = (Point)(v.remove(v.size()-1)); // lscht das letzte Element und bergibt es an p
				f = World.getField(p.x, p.y);
				a = f.getAgent();
				if (a == null || (a!=null && !a.isAlive())){
					name = null;
				}
				else{
					name = a.getName();
					if (i>0){
						agentsInRange.add(a);
						if (a.isCoordinator()){
							coordinatorInRange = true;
							if (bestCoordinatorValue < 0){
								bestCoordinatorValue = 0;
								bestCoordinator = a;
							}
						}
					}
				}
				memory.updateField(p.x, p.y, f.getFood(), a, name, World.round);
			}
		}
	}
	
	
	/**
	 * Fhrt eine Entscheidung herbei, was in dieser Runde zu tun ist.
	 * Die Hauptarbeit erledigt sat (Satisfaction).
	 * @return Liefert die Aktion, die zu tun ist.
	**/
	public int decide(){
		int action;

		sat.calcEstimatedSatis();
		action = sat.getAction();

		if (action == Satisfaction.MOVE){
			whereToMove = sat.getWhereToMove();
		}

		return(action);
	}
	
	/**
	 * Der Agent bewegt sich auf die parametrisierten Koordinaten.
	 * @param where ein Punkt mit den absoluten Koordinaten des Zielfeldes.
	**/
	public void move(Point where){
		if (!(World.getField(where.x, where.y).hasLivingAgent())){
			World.getField(x, y).setAgent(null);
			World.getField(where.x, where.y).setAgent(this);
			x = where.x;
			y = where.y;
			if (target != null && World.distance(x, y, target.x, target.y)<=1){
				target = null; //Ziel praktisch erreicht
			}
		}
		else {
			// dann ist was schiefgelaufen
		}
	}
	
	/**
	 * Der Agent tut einfach nix. 
	**/
	public void rest(){
	}
	
	/**
	 * Der Agent sammelt Nahrung. D.h. er entnimmt dem
	 * Nahrungsaufkommen des Feldes auf dem er steht einen Teil
	 * und fuegt ihn seinem Nahrungsvorrat hinzu.
	**/
	public void gather(){
		if (World.getField(x, y).getFood() > gatherRest){
			float amount = Math.min(gatherAmount, World.getField(x, y).getFood() - gatherRest);
			World.getField(x, y).setFood(World.getField(x, y).getFood() - amount);
			if (coordinator != null){
				float contribution = amount * World.getFloat("Agent.coordinatorContribution");
				coordinator.giveFood(contribution);
				amount = amount - contribution;
			}
			food = food + amount;
		}
		else{
			System.out.println("in Agent.gather: Aufruf in falscher Situation");
		}
	}
	
	/**
	 * Der Agent pflanzt sich fort. Auf einem mglichst nahegelegenen
	 * Feld schafft der Agent einen neuen, und gibt ihm einen Teil seines 
	 * Nahrungsvorrates ab.
	**/
	public void breed(){ //evtl. besseren Namen finden
		Point f = World.getFreeField(x, y); //Startfeld fr den neuen Agenten
		if (f == null){
			return;	// Pech, kein Platz mehr
		}
		if (World.sound) World.birthSound.play();
		Agent a = new Agent(this);
		this.numOfChildren++;
		a.setName(this.getName() + "/" + numOfChildren);
		a.setFood(childFood);
		a.setX(f.x);
		a.setY(f.y);
		mutateAndSetParameters(a);
		World.getField(f.x, f.y).setAgent(null);
		World.getField(f.x, f.y).setAgent(a);
		World.addAgent(a); 
		World.worldView.paintField(f.x, f.y);
		
		this.food = this.food - childFood;
		sat.setSatis(Satisfaction.BREED, 1f);
		AgentTree.newAgent(this);
	}
	
	/**
	 * Anfrage an den Koordinator, der fr den sendenden Agenten das optimale Zielfeld bestimmt.
	 * @param   client  der Agent, fr den das Ziel bestimmt ist.
	 * @param   range  seine Reichweite, innerhalb derer das Ziel liegen muss.
	 * @return     das Zielfeld (d.h. seine Koordinaten als Point).
	 */
	public Point getTarget(Agent client, int range){
		Point maxPoint = null;
		float maxValue = Float.NEGATIVE_INFINITY; //das sollte gengen :-)
		int maxRange = Math.min(range, Math.max(World.getInt("World.fieldsX"), World.getInt("World.fieldsY")) / 2);
		Vector v;
		Point p;
		float f;
		int r, i;
		float d;
	
		memory.merge(client.getMemory());
		if (range < 1){ //is'n bisschen wenig ...
			return(new Point(client.getX(), client.getY()));
		}
		for (r=1; r<=maxRange; r++){
			v = World.getCoordinatesInDistance(client.getX(), client.getY(), r);
			for (i=0; i<v.size(); i++){
				p = (Point)v.elementAt(i);
				if (!isTarget(p)){
					f = memory.evaluateTarget(p) * (1f - (float)(r-1) / (float)range);
					if (f > maxValue){
						maxValue = f;
						maxPoint = p;
					}
				}
			}
		}
		if (maxPoint == null) System.out.println("Error in Agent.getTarget"); //h?
		return(maxPoint);
		//ToDo das Zielfeld mit dem Agenten zusammen speichern
	}
	
	/** Hiermit ordnet sich der Agent dem Koordinatir unter, den er sich ausgesucht hat. */
	private void subordinate(){
		if (bestCoordinator == null){
			System.out.println("in Agent.subordinate: bestCoordinator = null??");
		}
		else{
			bestCoordinator.subordinateToCoordinator(this);
			subordinationDate = World.round;
			subordinationStartValue = food;
			coordinator = bestCoordinator;
		}
	}
	
	/** Diese Methode wird an den Koordinator geschickt, um die Unterordnung perfekt zu machen.
	 *  @param sub der Agent der sich unterordnet.
	 */
	public void subordinateToCoordinator(Agent sub){
		subordinates.add(sub);
		targets.add(new Point(-1, -1));
	}
	
	/** Hiermit beendet ein Untergebener diesen Zustand (der des Koordiniertwerdens)*/
	private void unsubordinate(){
		float value = food - subordinationStartValue;
		int i= knownCoordinators.indexOf(coordinator);
		if (i==-1){ //noch nicht in der Liste
			knownCoordinators.add(coordinator);
			knownCoordinatorsValues.add(new Float(value));
		}
		else{ //schon bekannt
			knownCoordinatorsValues.setElementAt(new Float(value), i);
		}
		coordinator.unsubordinateFromCoordinator(this);
		coordinator = null;
	}
	
	/** Hier wieder das Pendant auf Seiten des Koordinators, dem mitgeteilt wird, dass ein Untergebener die Beziehung beendet.
	 * @param sub der Agent, der die Unterordnung beendet.
	 */
	public void unsubordinateFromCoordinator(Agent sub){
		int i = subordinates.indexOf(sub);
		subordinates.remove(i);
		targets.remove(i);
	}
	
	/** Der Agent wechselt in den Status eines Koordinators. */
	private void startCoordinating(){
		startCoordinatingDate = World.round;
		isCoordinator = true;
	}
	
	/** Der Agent beendet seinen Status als Koordinator */
	private void endCoordinating(){
		isCoordinator = false;
		for (int i=0; i<subordinates.size(); i++){
			((Agent)subordinates.elementAt(i)).coordinationTerminated();
		}
		subordinates.removeAllElements();
		targets.removeAllElements();
	}
	
	/** Hiermit wird einem Untergebenen das Ende der Koordination mitgeteilt. */
	public void coordinationTerminated(){
		float value = food - subordinationStartValue;
		int i= knownCoordinators.indexOf(coordinator);
		if (i==-1){ //noch nicht in der Liste
			knownCoordinators.add(coordinator);
			knownCoordinatorsValues.add(new Float(value));
		}
		else{ //schon bekannt
			knownCoordinatorsValues.setElementAt(new Float(value), i);
		}
		coordinator = null;
	}		
	/** Hiermit wird einem Untergebenen der Tod seines Koordinators mitgeteilt. */
	public void coordinatorDied(){
		int i = knownCoordinators.indexOf(coordinator);
		if (i>=0){
			knownCoordinators.remove(i);
		}
		coordinator = null;
	}		

	/**
	 * Die Parameter, die der Mutation unterliegen werden im Rahmen des Modells mutiert und dem neuen Agenten zugewisen.
	 * Als Basis dienen die Werte des Elternagenten.
	 * @param child der Nachwuchs, dem die neuen Werte zugewiesen werden.
	 */
	private void mutateAndSetParameters(Agent child){
		float eatFoodPRM = World.mutate2(World.getFloat("Agent.eatFoodPR.min"),
																		World.getFloat("Agent.eatFoodPR.max"),
																		eatFoodPR,
																		World.getFloat("Agent.eatFoodPR.mutable"));
																		
		float pubertyAgeM = World.mutate2(World.getFloat("Agent.pubertyAge.min"),
																		 World.getFloat("Agent.pubertyAge.max"),
																		 pubertyAge,
																		 World.getFloat("Agent.pubertyAge.mutable"));
																		
		float gatherAmountM = World.mutate2(World.getFloat("Agent.gatherAmount.min"),
																				World.getFloat("Agent.gatherAmount.max"),
																				gatherAmount,
																				World.getFloat("Agent.gatherAmount.mutable"));
																				
		float gatherRestM = World.mutate2(World.getFloat("Agent.gatherRest.min"),
																			World.getFloat("Agent.gatherRest.max"),
																			gatherRest,
																			World.getFloat("Agent.gatherRest.mutable"));
																		
		float childFoodM = World.mutate2(World.getFloat("Agent.childFood.min"),
																		World.getFloat("Agent.childFood.max"),
																		childFood,
																		World.getFloat("Agent.childFood.mutable"));
																		
		float memoryMaxAgeM = World.mutate2(World.getFloat("Agent.memoryMaxAge.min"),
																			 World.getFloat("Agent.memoryMaxAge.max"),
																			 memoryMaxAge,
																			 World.getFloat("Agent.memoryMaxAge.mutable"));
																		
		float viewingRangeM = World.mutate2(World.getFloat("Agent.viewingRange.min"),
																		World.getFloat("Agent.viewingRange.max"),
																		viewingRange,
																		World.getFloat("Agent.viewingRange.mutable"));
																		
		float planningRangeM = World.mutate2(World.getFloat("Agent.planningRange.min"),
																				World.getFloat("Agent.planningRange.max"),
																				planningRange,
																				World.getFloat("Agent.planningRange.mutable"));
																		
		float survivalWeightM = World.mutate2(World.getFloat("Agent.survivalWeight.min"),
																				 World.getFloat("Agent.survivalWeight.max"),
																				 survivalWeight,
																				 World.getFloat("Agent.survivalWeight.mutable"));
																		
		float wealthWeightM = World.mutate2(World.getFloat("Agent.wealthWeight.min"),
																			 World.getFloat("Agent.wealthWeight.max"),
																			 wealthWeight,
																			 World.getFloat("Agent.wealthWeight.mutable"));
																		
		float curiosityWeightM = World.mutate2(World.getFloat("Agent.curiosityWeight.min"),
																					World.getFloat("Agent.curiosityWeight.max"),
																					curiosityWeight,
																					World.getFloat("Agent.curiosityWeight.mutable"));
																		
		float breedingWeightM = World.mutate2(World.getFloat("Agent.breedingWeight.min"),
																				 World.getFloat("Agent.breedingWeight.max"),
																				 breedingWeight,
																				 World.getFloat("Agent.breedingWeight.mutable"));

		float influenceWeightM = World.mutate2(World.getFloat("Agent.influenceWeight.min"),
																				  World.getFloat("Agent.influenceWeight.max"),
																				  influenceWeight,
																				  World.getFloat("Agent.influenceWeight.mutable"));
		
		//Gewichte auf die Summe 1 normen
		float sumWeights = survivalWeightM + wealthWeightM + curiosityWeightM + breedingWeightM + influenceWeightM;
		survivalWeightM = survivalWeightM / sumWeights;
		wealthWeightM = wealthWeightM / sumWeights;
		curiosityWeightM = curiosityWeightM / sumWeights;
		breedingWeightM = breedingWeightM / sumWeights;
		influenceWeightM = influenceWeightM / sumWeights;
		
		child.setParameters(eatFoodPRM,
												pubertyAgeM,
												gatherAmountM,
												gatherRestM,
												childFoodM,
												memoryMaxAgeM,
												viewingRangeM,
												planningRangeM,
												survivalWeightM,
												wealthWeightM,
												curiosityWeightM,
												breedingWeightM,
												influenceWeightM);
	}

	/**
	 * Liefert ein Array mit den potentiell durch Mutation vernderlichen Werten des Agenten.
	 * return array mit 13 float Werten.
	**/
	public float[] getMutableValues(){
		float[] result = {eatFoodPR, pubertyAge, gatherAmount, gatherRest, childFood, 
												memoryMaxAge, viewingRange, planningRange, survivalWeight, 
												wealthWeight, curiosityWeight, breedingWeight, influenceWeight};
		return(result);
	}


	/**
	 * Der Agent gibt die Farbe zurck, in der er gerne gezeichnet wrde.
	 * @return die Farbe.
	**/
	public Color getColor(){
		if (food <= 0)
			return(Color.black);
		else{
			float value = Math.max(10f, food);	// nicht kleiner 10
			float max = 2 * World.getFloat("World.averageFoodOnField");
			value = Math.min(max, value);		// nicht groesser doppelter average-Wert
			return(new Color(1f - value / max, 1f - value / max, 1f));
		}
	}

	/**
	 * Lebt der Agent noch?
	 * @return  ein boolean, der angibt, ob der Agent noch lebt.
	**/
	public boolean isAlive(){
		return(deathDay == -1);
	}

	/**
	 * Ist der Agent zur Zeit im "Koordiniermodus".
	 * @return true, falls der Agent koordiniert, false wenn nicht.
	**/
	public boolean isCoordinator(){
		return(isCoordinator);
	}
	

	/**
	 * Liefert die Zahl der Untergebenen.
	 * @return die Anzahl der von diesem Agenten koordinierten Agenten.
	 */
	public int getNumOfSubordinates(){
		return(subordinates.size());
	}

	/**
	 * Liefert den Koordinator des Agenten zurck, null falls keiner existiert.
	 * @return der Koordinator des Agenten.
	 */
	public Agent getCoordinator(){
		return(coordinator);
	}

	/**
	 * Liefert die Bewertung des besten Koordinators den der Agent bisher hatte.
	 * @return  die Bewertung. (deafult fr unbekannte Koordinatoren ist 0)
	 */
	public float getBestCoordinatorValue(){
		return(bestCoordinatorValue);
	}
	/**
	 * Die aktuelle Bewertung des aktuellen Koordinators des Agenten.
	 * @return die Bewertung
	 */
	public float getCoordinatorValue(){
		return(food - subordinationStartValue);
	}
	/**
	 * Liefert das aktuelle Zielfeld des Agenten, oder null falls er nicht koordiniert wird oder schon am Ziel ist.
	 * @return die Koordinaten des Zielfeldes als Point
	 */
	public Point getTarget(){
		return(target);
	}
	/**
	 * Hat der Agent einen Koordinator oder nicht?
	 * @return true wenn ja, false falls nicht.
	 */
	public boolean hasCoordinator(){
		return(coordinator != null);
	}
	/**
	 * Hat der Agent ein Zielfeld?
	 * @return true oder false (siehe getTarget)
	 */
	public boolean hasTarget(){
		return(target != null);
	}
	/**
	 * Liefert true, falls das Feld Zielfeld eines Untergebenen des Koordinators ist, false sonst.
	 * @param die Koordinaten des fraglichen Feldes als Point
	 */
	public boolean isTarget(Point p){
		for (int i=0; i<targets.size(); i++){
			if (((Point)targets.elementAt(i)).equals(p)){
				return(true);
			}
		}
		return(false);
	}
	
	/**
	 * Soll der Agent dargestellt werden?
	 * @return  ein boolean, der angibt, ob der Agent noch lebt, oder erst krzlich verstorben ist.
	**/
	public boolean isVisible(){
		if (deathDay == -1){
			return(true);
		}
		else{
			return(World.round - deathDay < 10);
		}
	}
	
	/**
	 * Liefert das maximale Alter von Informationen im Memory, danach werden diese vergessen.
	 * @return Wert des Parameters memoryMaxAge
	 */
	public float getMemoryMaxAge(){
		return memoryMaxAge;
	}
	
	/**
	 * Liefert die Nahrungsmenge, die ein Agent maximal pro Runde sammelt.
	 * @return Wert des Parameters gatherAmount
	 */
	public float getGatherAmount(){
		return gatherAmount;
	}
	
	/**
	 * Liefert die Nahrungsmenge, die ein Agent mindestens auf einem Feld belsst.
	 * @return Wert des Parameters gatherRest
	 */
	public float getGatherRest(){
		return gatherRest;
	}
	
	/**
	 * Liefert die Werte fr die Gewichtungen der Bedrfnisse untereinander.
	 * @return ein Array mit den Gewichten von {survival, wealth, curiosity, breeding, influence}
	 */
	public float[] getWeights(){
		float[] weights = {survivalWeight, wealthWeight, curiosityWeight, breedingWeight, influenceWeight};
		return(weights);
	}

	/**
	 * Legt die individuellen Werte fr die Parameter des Agenten fest.
	 *
	 * @param   eatFoodPR  soviel Nahrung verbraucht der Agent pro Runde
	 * @param   pubertyAge  ab diesem Alter kann sich der Agent vermehren
	 * @param   gatherAmount  maximal diese Nahrungsmenge sammelt der Agent pro Runde
	 * @param   gatherRest  mindestens diese Nahrungsmenge belsst der Agent auf dem Feld
	 * @param   childFood  soviel Nahrung gibt der Agent an jeden Nachkommen weiter
	 * @param   memoryMaxAge  das maximale Alter von Informationem im Gedchtnis (Memory)
	 * @param   viewingRange  alle Felder bis zu diesem Abstand kann der Agent wahrnehmen
	 * @param   planningRange  alle Informationen ber Felder bis zu diesem Abstand flieen in die Planung ein
	 * @param   survivalWeight  Gewichtung des Bedrfnis berleben
	 * @param   wealthWeight  Gewichtung des Bedrfnis Reichtum
	 * @param   curiosityWeight  Gewichtung des Bedrfnis Neugier
	 * @param   breedingWeight  Gewichtung des Bedrfnis Fortpflanzung
	 * @param   influenceWeight  Gewichtung des Bedrfnis Einfluss
	 */
	public void setParameters(float eatFoodPR,
														float pubertyAge,
														float gatherAmount,
														float gatherRest,
														float childFood,
														float memoryMaxAge,
														float viewingRange,
														float planningRange,
														float survivalWeight,
														float wealthWeight,
														float curiosityWeight,
														float breedingWeight,
														float influenceWeight){
		this.eatFoodPR = eatFoodPR;
		this.pubertyAge = pubertyAge;
		this.gatherAmount = gatherAmount;
		this.gatherRest = gatherRest;
		this.childFood = childFood;
		this.memoryMaxAge = memoryMaxAge;
		this.viewingRange = viewingRange;
		this.planningRange = planningRange;
		this.survivalWeight = survivalWeight;
		this.wealthWeight = wealthWeight;
		this.curiosityWeight = curiosityWeight;
		this.breedingWeight = breedingWeight;
		this.influenceWeight = influenceWeight;
		sat.setWeights(survivalWeight, wealthWeight, curiosityWeight, breedingWeight, influenceWeight);
	}
	

	/**
	 * Setzt die individuellen Parameter auf die im Modell festgelegten Defaultwerte.
	 */
	public void setDefaultParameters(){
		eatFoodPR = World.getFloat("Agent.eatFoodPR.default");
		pubertyAge = World.getFloat("Agent.pubertyAge.default");
		gatherAmount = World.getFloat("Agent.gatherAmount.default");
		gatherRest = World.getFloat("Agent.gatherRest.default");
		childFood = World.getFloat("Agent.childFood.default");
		memoryMaxAge = World.getFloat("Agent.memoryMaxAge.default");
		viewingRange = World.getFloat("Agent.viewingRange.default");
		planningRange = World.getFloat("Agent.planningRange.default");
		survivalWeight = World.getFloat("Agent.survivalWeight.default");
		wealthWeight = World.getFloat("Agent.wealthWeight.default");
		curiosityWeight = World.getFloat("Agent.curiosityWeight.default");
		breedingWeight = World.getFloat("Agent.breedingWeight.default");
		influenceWeight = World.getFloat("Agent.influenceWeight.default");
		sat.setWeights(survivalWeight, wealthWeight, curiosityWeight, breedingWeight, influenceWeight);
	}
	
	/**
	 * Setzt den Namen des Agenten.
	 * @param name der neue Name des Agenten.
	**/
	public void setName(String name){
		this.name = name;
	}
	
	/**
	 * Liefert den Namen des Agenten.
	 * @return der Name des Agenten
	**/
	public String getName(){
		return(name);
	}
	
	/**
	 * Liefert die wesentlichen Informationen des Agenten 
	 * @return ein String, in dem zeilenweise ein paar Infos zum aktuellen Zustand des Agenten steht.
	 */
	public String getShortInfo(){ //ToDo1: ausbauen (evtl. zweierlei Infos anbieten: normal und extra)
		StringBuffer info = new StringBuffer();
		info.append("Name: " + name + "\n");
		info.append("(X, Y): (" + x + ", " + y +")\n");
		info.append("Food: " + food + "\n");
		info.append("DeathDay: ");
		if (deathDay == -1){
		 	info.append("alive\n");
		}
		else{
			info.append(deathDay + "\n");
		}
		info.append("Age: " + age + "\n");
		info.append("Children: " + numOfChildren);
		
		return(info.toString());
	}

	/** Liefert etwas ausfhrlichere Informationen zum Agenten */
	public String getMediumInfo(){
		StringBuffer info = new StringBuffer(getShortInfo());
		info.append("\n");
		info.append("BirthDay: " + birthDay + "\n");
		info.append("Coordinator: ");
		if (coordinator != null){
			info.append(coordinator.getName() + "\n");
		}
		else{
			info.append("none \n");
		}
		info.append("Is Coordinator: ");
		info.append(isCoordinator +"\n");
		info.append("Target: ");
		if (target != null){
			info.append("("+target.x+","+target.y+")\n");
		}
		else{
			info.append("none \n");
		}
		info.append("Subordinates: ");
		for (int i=0; i<subordinates.size(); i++){
			info.append(((Agent)subordinates.elementAt(i)).getName() + " ");
		}
		info.append("\n");
		info.append("LastAction: " + lastAction);

		return(info.toString());
	}
	
	/** Liefert ausfhrliche Informationen zum Agenten */
	public String getLongInfo(){
		StringBuffer info = new StringBuffer(getMediumInfo());
		info.append("\n");
		info.append("EatFoodPR: " + eatFoodPR + "\n");
		info.append("GatherAmount: " + gatherAmount + "\n");
		info.append("PubertyAge: " + pubertyAge + "\n");
		info.append("ChildFood: " + childFood + "\n");
		info.append("GatherRest: " + gatherRest + "\n");
		info.append("ViewingRange: " + viewingRange + "\n");
		info.append("MemoryMaxAge: " + memoryMaxAge + "\n");
		info.append("PlanningRange: " + planningRange + "\n");
		info.append("SurvivalWeight: " + survivalWeight + "\n");
		info.append("BreedingWeight: " + breedingWeight + "\n");
		info.append("WealthWeight: " + wealthWeight + "\n");
		info.append("CuriosityWeight: " + curiosityWeight + "\n");
		info.append("InfluenceWeight: " + influenceWeight);

		return(info.toString());
	}

	/** Liefert frs Debuggen hilfreiche Informationen zum Agenten */
	public String getDebugInfo(){//ToDo implementiern
		StringBuffer info = new StringBuffer(getMediumInfo());
		info.append("\n");
		info.append("EstimSatisGather: ");
		info.append(sat.getEstSatis(Satisfaction.GATHER));
		info.append("\n");
		info.append("EstimSatisMove: ");
		info.append(sat.getEstSatis(Satisfaction.MOVE));
		info.append("\n");
		info.append("EstimSatisBreed: ");
		info.append(sat.getEstSatis(Satisfaction.BREED));
		info.append("\n");
		info.append("EstimSatisStartCoord: ");
		info.append(sat.getEstSatis(Satisfaction.START_COORDINATING));
		info.append("\n");
		info.append("EstimSatisStartSub: ");
		info.append(sat.getEstSatis(Satisfaction.SUBORDINATE));
		info.append("\n");
		info.append("EstimSatisEndCoord: ");
		info.append(sat.getEstSatis(Satisfaction.END_COORDINATING));
		info.append("\n");
		info.append("EstimSatisUnSub: ");
		info.append(sat.getEstSatis(Satisfaction.UNSUBORDINATE));
		info.append("\n");
		info.append("EstimSatisRest: ");
		info.append(sat.getEstSatis(Satisfaction.REST));
		return(info.toString());
	}

	/**
	 * Liefert die Nummer der Runde, in der der Agent geboren (oder geschaffen) wurde
	 * @return der Geburtstag des Agenten
	 */
	public int getBirthDay(){
		return(birthDay);
	}
	
	/**
	 * Legt die Runde fest, in der der Agent gestorben ist.
	 * @param der Todestag des Agenten
	 */
	public void setDeathDay(int dd){
		deathDay = dd;
	}
	
	/**
	 * Liefert die Nummer der Runde, in der der Agent gestorben ist.
	 * @return der Todestag (-1 falls der Agent noch lebt)
	 */
	public int getDeathDay(){
		return(deathDay);
	}
		
	/**
	 * Liefert das Gedchtnis des Agenten.
	 * @return das Gedchtnis des Agenten.
	**/
	public Memory getMemory(){
		return(memory);
	}
	
 /**
  * Legt die X-Koordinate des Agenten fest.
  * @param x (0 <= x < fieldsX)
 **/
	public void setX(int x){
		this.x = x;
	}
	
 /**
  * Liefert die aktuelle X-Koordinate das Agenten.
  * @return x
  */
	public int getX(){
		return(x);
	}

 /**
  * Legt die Y-Koordinate des Agenten fest.
  * @param y (0 <= y < fieldsY)
 **/
	public void setY(int y){
		this.y = y;
	}

 /**
  * Liefert die aktuelle Y-Koordinate das Agenten.
  * @return y
  */
	public int getY(){
		return(y);
	}
	
	/** Setzt den Nahrungsvorrat des Agenten auf den bergebenen Wert */
	public void setFood(float food){
		this.food = food;
	}
	
	/** Erlaubt dem Empfnger die bergebene nahrungsmenge seinem Vorrat hinzuzufgen.
	 *  Der Absender muss dafr sorgen, die Menge bei sich abzuziehen. 
	 */
	public void giveFood(float amount){
		food = food + amount;
	}	
	
	/** Liefert den aktuellen Nahrungsvorrat des Agenten zurck */
	public float getFood(){
		return(food);
	}
	
	/** Liefer den Wert des Parameters eatFoodPR (Nahrungsverbrauch pro Runde). */
	public float getEatFoodPR(){
		return(eatFoodPR);
	}
	
	/** Liefert den Wert des Parameters viewingRange (Sichtweite) */
	public int getViewingRange(){
		return((int)viewingRange);
	}
	
	/** Liefert den Wert des Parameters planningRange ("geistige Sichtweite") */
	public int getPlanningRange(){
		return((int)planningRange);
	}
	
	/** Legt das Alter des Agenten fest. */
	public void setAge(int a){
		age = a;
	}
	
	/** Liefert das Alter des Agenten zurck. */
	public int getAge(){
		return(age);
	}
	
	/** Liefert den Wert des Parameters childFood (Nahrungmenge fr den Nachwuchs) */
	public int getChildFood(){//ToDo int?
		return((int)childFood);
	}
	
	/** Liefert den Wert des Parameters pubertyAge (ab dann ist der Agent fruchtbar) */
	public int getPubertyAge(){
		return((int)pubertyAge);
	}
	
	/** Liefert die Zahl der Kinder des Agenten. */
	public int getNumOfChildren(){
		return(numOfChildren);
	}
	
	/** ffnet ein Fenster, um den Gedchtnisinhalt des Agenten anzuzeigen. */
	public boolean setMemoryMonitor(){
		if (memoryMonitor != null){
			memoryMonitor.setState(Frame.NORMAL);
			memoryMonitor.show();
			memoryMonitor.update();
			return(false);
		}
		else{
			memoryMonitor = new MemoryMonitor(this);
			memoryMonitor.show();
			memoryMonitor.update();
			return(true);
		}
	}

	/** ffnet ein Fenster, um den aktuelle Informationen des Agenten anzuzeigen. */
	public boolean setAgentMonitor(){
		if (agentMonitor != null){
			agentMonitor.setState(Frame.NORMAL);
			agentMonitor.update();
			agentMonitor.show();
			return(false);
		}
		else{
			agentMonitor = new AgentMonitor(this);
			agentMonitor.update();
			agentMonitor.show();
			return(true);
		}
	}
	
	/** Schliesst den MemoryMonitor des Agenten. */
	public void removeMemoryMonitor(){
		if (memoryMonitor != null){
			memoryMonitor.dispose();
			memoryMonitor = null;
		}
	}
	
	/** Schliesst den AgentMonitor des Agenten. */
	public void removeAgentMonitor(){
		if (agentMonitor != null){
			agentMonitor.dispose();
			agentMonitor = null;
		}
	}
	
	/** Bringt eventuell vorhandene MonitorFenster des Agenten in den Vordergrund. */
	public void showMonitors(){
		if (agentMonitor != null){
			agentMonitor.setState(Frame.NORMAL);
			agentMonitor.show();
		}
		if (memoryMonitor != null){
			memoryMonitor.setState(Frame.NORMAL);
			memoryMonitor.show();
		}
	}
	
	/** Liefert das Satisfaction-Objekt des Agenten zurck.*/
	public Satisfaction getSatisfaction(){
		return(sat);
	}
	
	/** Sagt, welche Aktion der Agent gerade getan hat.*/
	public String getLastAction(){
		return(lastAction);
	}
	
	/** Liefert den Namen des Agenten.*/
	public String toString(){
		return(name);
	}
	
	public void removeAgent(){
		Agent child;
		for (int i=0; i<children.size(); i++){
			child = (Agent)(children.elementAt(i));
			child.setParent(parent); // der eigenen Elternagent wird neuer Vater der Kinder
			parent.insert(child, 0);  // die neuen Kinder werden beim Elternagenten eingetragen
		}
		parent.remove(this);
		memory = null;
		sat = null;
		children = null;
		parent = null;
		agentsInRange = null;
	}
	
	/**
	 * Liefert eine Aufzhlung aller direkten Nachkommen.
	 * @return die direkten Nachkommen des Agenten
	 * 
	 * Diese und die folgenden Methoden stammen aus dem Interface MutableTreeNode.
	 * Es sind nur diejenigen ausprogrammiert, die auch tatschlich bentigt werden.
	 */
	public Enumeration children() {
		return(children.elements());
	}
	
	/**
	 * Liefert zurck, ob es dem Agenten erlaubt ist Nachwuchs in die Welt zu setzen.
	 * @return true, da prinzipiell jeder Agent Nachwuchs bekommen kann.
	 */
	public boolean getAllowsChildren() {
		return(true);
	}
	
	/**
	 * Liefert das Kind mit dem bergebenen Index.
	 * @param i der Index des Gesuchten Kindes
	 * @return das (i+1)te Kind des Agenten
	 */
	public TreeNode getChildAt(int i) {
		return((TreeNode)children.elementAt(i));
	}
	/** Liefert die Zahl der Kinder. */
	public int getChildCount() {
		return(children.size());
	}
	/** Liefert den Index des bergebenen Kindes. */
	public int getIndex(TreeNode child) {
		return(children.indexOf(child));
	}
	/** Liefert den Elternagenten. */
	public TreeNode getParent() {
		return((TreeNode)parent);
	}
	/** Ist der Agent ein Blatt im AgentTree (hat er Kinder).*/
	public boolean isLeaf() {
		return(children.size() == 0);
	}
	/** Trgt ein neues Kind beim Agenten ein.*/
	public void insert(MutableTreeNode child, int index) {
		children.add(index, child);
	}
	/** nicht implementiert */
	public void remove(int index) {} //Removes the child at index from the receiver. 
	/** nicht implementiert */
	public void remove(MutableTreeNode node) {
		children.remove(node);
	} //Removes node from the receiver. 
	/** nicht implementiert */
	public void removeFromParent() {} //Removes the receiver from its parent. 
	/** legt einen neuen Elternknoten fest */
	public void setParent(MutableTreeNode newParent) {
		parent = (Agent)newParent;
	}
	/** nicht implementiert */
	public void setUserObject(Object object) {} //Resets the user object of the receiver to object. 
}