import java.awt.*;
import java.awt.event.*;

/** Bietet ein Fenster mit Informationen zu einem aktuellen Agenten.
 *  Es gibt die Mglichkeit zwischen drei verschiedenen Detailierungsgraden zu whlen.
 *  AgentMonitor verzischtet (im Gegensatz zu AgentTree) auf die Verwendung von
 *  swing-Klassen, und sollte daher weniger Probleme bereiten.
 */
public class AgentMonitor extends Frame implements WindowListener, ItemListener{
	/** Referenz auf den zugehrigen Agenten */
	private Agent agent;
	/** Hier steht dann die Information */
	private TextArea textArea;
	/** Dient der Zwischenspeicherung der Infos */
	private String agentInfo;
	/** Dient der Auswahl des Detaillierungsgrades */
	private CheckboxGroup cbg;
	/** Dient ebenfalls der Auswahl des Detaillierungsgrades */
	private Checkbox shortCb, mediumCb, longCb, debugCb;
	
	/** Der Konstruktor
	 *  @param a der Agent zu dem der Monitor gehrt
	 */
	public AgentMonitor(Agent a){
		agent = a;
		textArea = new TextArea(agent.getShortInfo(), 10, 30, TextArea.SCROLLBARS_BOTH);
		setLayout(new BorderLayout());
		add(textArea, BorderLayout.CENTER);
		
		Panel panel = new Panel();
		panel.setLayout(new FlowLayout());
		cbg = new CheckboxGroup();
		shortCb = new Checkbox("short", cbg, true);
		shortCb.addItemListener(this);
		mediumCb = new Checkbox("medium", cbg, false);
		mediumCb.addItemListener(this);
		longCb = new Checkbox("long", cbg, false);
		longCb.addItemListener(this);
		debugCb = new Checkbox("debug", cbg, false);
		debugCb.addItemListener(this);
		panel.add(shortCb);
		panel.add(mediumCb);
		panel.add(longCb);
		if (World.debug) panel.add(debugCb);
		
		add(panel, BorderLayout.NORTH);		
		addWindowListener(this);
		pack();
	}
	
	public void update(){
		Graphics g = getGraphics();
		setTitle("Agent: " + agent.getName());
		if (cbg.getSelectedCheckbox() == shortCb){
			agentInfo = agent.getShortInfo();
		}
		else if (cbg.getSelectedCheckbox() == mediumCb){
			agentInfo = agent.getMediumInfo();
		}
		else if (cbg.getSelectedCheckbox() == longCb){
			agentInfo = agent.getLongInfo();
		}
		else if (cbg.getSelectedCheckbox() == debugCb){
			agentInfo = agent.getDebugInfo();
		}
		textArea.setText(agentInfo);
	}
	
	/** Wenn der Benutzer das Fenster schliesst wird der Monitor entfernt. */
	public void windowClosing(WindowEvent e){
		agent.removeAgentMonitor();
		dispose();
	}
	public void windowOpened(WindowEvent e){}
	public void windowClosed(WindowEvent e){}
	public void windowIconified(WindowEvent e){}
	public void windowDeiconified(WindowEvent e){}
	public void windowActivated(WindowEvent e){}
	public void windowDeactivated(WindowEvent e){}
	
	/**
	 * Wird aktiviert, wenn eine Checkbox geklickt wird.
	 */
	public void itemStateChanged(ItemEvent e){
		update();
	}
}