import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

/**
 * Diese Klasse stellt ein eigenes Fenster zur Verfgung, um alle Agenten
 * als Baum darzustellen. Gegliedert wird nach den Verwandschaftsverhltnissen.
 * Im unteren Teilfenster erscheinen zum ausgewhltenAgenten ausfhrliche Informationen.
 */
public class AgentTree extends JFrame implements TreeSelectionListener, WindowListener{
	
	/** Der Rahmen fr die beiden Teilfenster */
	private JSplitPane splitPane;
	/** Der scrollbare Bereich fr den Baum */
	private JScrollPane treeView;
	/** Der scrollbare Hintergrund fr die Agenteninformationen */
	private JScrollPane agentPane;
	/** Hier werden Informationen zum ausgewhlten Agenten angezeigt */
	private JTextArea agentView;
	/** Der  Baum */
	private JTree tree;
	/** Die Wurzel (wird nicht angezeigt)*/
	private TreeNode root;
	/** Das TreeModel zum Baum */
	private DefaultTreeModel treeModel;
	/** Der momentan ausgewhlte Agent */
	private static Agent selectedAgent = null;
	/** Statische Referenz auf den AgentTree (falls vorhanden)*/
	private static AgentTree visibleTree = null;
	
	
	/** Der Konstuktor erschafft einen einzigen AgentTree */
	public AgentTree(){
		super("Agents");
		if (visibleTree == null) {
			visibleTree = this;
			treeModel = new DefaultTreeModel(World.getAgentTreeRoot());
			tree = new JTree(treeModel);
			tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
			tree.addTreeSelectionListener(this);
			tree.setRootVisible(false);
			tree.setShowsRootHandles(true);
			tree.putClientProperty("JTree.lineStyle", "Angled");
			treeView = new JScrollPane(tree);
			treeView.setPreferredSize(new Dimension(200, 150));
			agentView = new JTextArea();
			agentPane = new JScrollPane(agentView);
			splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, treeView, agentPane);
			getContentPane().add(splitPane);
			setSize(250, 700);
			addWindowListener(this);
			show();
		}
		else{
			//es kann nur einen geben...
		}
	}
	
	/** Wenn sich die Informationen zum ausgewhlten Agenten ndert,
	 * werden diese aktualisiert.
	 */
	public static void update(Agent agent){
		if (visibleTree != null && selectedAgent == agent){
			visibleTree.agentView.setText(selectedAgent.getLongInfo());//ToDo gewnschte Infomenge anzeigen
		}
	}
	
	/**
	 * Hier wird die Struktur aufmerksam gemacht, dass es einen neuen Agenten gibt.
	 */
	public static void newAgent(Agent parent){
		if (visibleTree != null){
			visibleTree.treeModel.nodeStructureChanged((TreeNode)parent);
		}
	}

	/**
	 * Fr das Interface TreeSelectionListener. 
	 * Wird aufgerufen, wenn vom Benutzer in der Baumanzeige
	 * ein anderer Agent ausgewhlt wird.
	 */
	public void valueChanged(TreeSelectionEvent e) {
		selectedAgent = (Agent)(e.getPath().getLastPathComponent());
		agentView.setText(selectedAgent.getLongInfo()); //ToDo gewnschte Infomenge anzeigen
	
		Point position = new Point(selectedAgent.getX(), selectedAgent.getY());
		if (World.getEditMode()){
			World.worldView.getTorusView().hideCursor();
			World.setCursorPosition(position);
			World.worldView.getTorusView().showCursor();
		}
	}

	/** Wenn der Benutzer das Fenster schliesst, werden die statischen Referenzen
	 * wieder zurckgesetzt.
	 */
	public void windowClosing(WindowEvent e){
		visibleTree = null;
		selectedAgent = null;
	}
	/** nicht implementiert */
	public void windowOpened(WindowEvent e){}
	/** nicht implementiert */
	public void windowClosed(WindowEvent e){}
	/** nicht implementiert */
	public void windowIconified(WindowEvent e){}
	/** nicht implementiert */
	public void windowDeiconified(WindowEvent e){}
	/** nicht implementiert */
	public void windowActivated(WindowEvent e){}
	/** nicht implementiert */
	public void windowDeactivated(WindowEvent e){}
}