import java.io.*;
import java.awt.Color;

/**
 * Repraesentiert ein einzelnes Feld in der Welt.
 */
public class Field implements Serializable{
	private float food;
	private boolean fertile;
	private Agent agent = null;
	
	/**
	 * Der Kontruktior setzt die Defaultwerte.
	 * 0 fuer Nahrungsmenge (food) und false fuer die 
	 * Fruchtbarkeit.
	 */
	public Field(){
		this(0, false);
	}
	
	/**
	 * Dieser Kontruktor setzt die uebergebenen Werte.
	 * @param food die Menge der auf das Feld zu platzierende Nahrung.
	 * @param gf die zugeordnete Wachstumskurve (GrowFunction) bzw. null.
	**/
	public Field(float food, boolean fertile){
		this.food = food;
		this.fertile = fertile;
	}
	
	/**
	 * Liefert die Menge der auf dem Feld vorhandenen Nahrung.
	 * @return ein positiver int-Wert fr die Nahrungsmenge.
	**/
	public float getFood(){
		return(food);
	}
	
	/**
	 * Setzt die Menge der auf dem Feld vorhandenen Nahrung.
	 * @param food ein positiver float-Wert fr die Nahrungsmenge.
	**/
	public void setFood(float food){
		boolean f = fertile;
		this.food = Math.max(food, 0);
		fertile = (food >= 0);
		if (f && !fertile){
			System.out.print("Field.setFood has changed fertility");
			System.out.println(food);
		}
	}
	
	public void setFerility(boolean ferile){
		this.fertile = fertile;
	}
	public boolean getFertility(){
		return(fertile);
	}
	
	public boolean grow(){
		if (fertile){
			float foodInc = GrowFunction.foodInc(food);
			food = food + foodInc;
			return(foodInc != 0f);
		}
		else
			return(false);
	}
	/**
	 * Liefert die Farbe, fr die Darstellung des Feldes
	 * @return die Farbe, die das Feld zur eigenen Darstellung fr angemessen hlt.
	**/
	public Color getColor(){
		float max = 2f * World.getFloat("World.averageFoodOnField"); //doppelter average Wert als oberter angezeigter
		float value = Math.max(0f, food);	// nicht kleiner 0
		value = Math.min(max, value);
		return(new Color(1f - value / max, 1f, 1f - value / max));
	}

	/**
	 * Setzt den Agenten auf das Feld.
	 * @param a der Agent der auf das Feld gesetzt werden soll.
	**/
	public void setAgent(Agent a){
		agent = a;
	}
	
	/**
	 * Liefert den Agenten zurck, der auf dem Feld sitzt, oder null.
	 * @return der Agent auf dem Feld - null falls keiner da ist.
	**/
	public Agent getAgent(){
		return(agent);
	}
	
	/**
	 * Sagt, ob ein Agent auf dem Feld ist oder nicht.
	 * @return true falls Agent auf dem Feld, false falls nicht. 
	**/
	public boolean hasAgent(){
		return(agent != null);
	}

	/**
	 * Sagt, ob das Feld von einem lebenden Agenten besetzt ist.
	 * @return true falls lebender Agent auf dem Feld, false falls nicht. 
	**/
	public boolean hasLivingAgent(){
		return((agent != null) && (agent.isAlive()));
	}
}