import java.io.*;
import java.awt.Color;
import java.awt.Point;

public class Memory implements Serializable{
	 
	private Agent agent;
	private MemoryField[][] fields;
	
	public Memory(Agent a){
		fields = new MemoryField[World.getInt("World.fieldsX")][World.getInt("World.fieldsY")];
		agent = a;
	}
	
	public MemoryField getField(int x, int y){
		if (x < 0) x += World.getInt("World.fieldsX");
		if (x >= World.getInt("World.fieldsX")) x -= World.getInt("World.fieldsX");
		if (y < 0) y += World.getInt("World.fieldsY");
		if (y >= World.getInt("World.fieldsY")) y -= World.getInt("World.fieldsY");
		return(fields[x][y]);
	}
	public boolean updateField(int x, int y, float food, Agent agent, String agentName, int date){
		if (x < 0) x += World.getInt("World.fieldsX");
		if (x >= World.getInt("World.fieldsX")) x -= World.getInt("World.fieldsX");
		if (y < 0) y += World.getInt("World.fieldsY");
		if (y >= World.getInt("World.fieldsY")) y -= World.getInt("World.fieldsY");
		if (fields[x][y] == null)
			fields[x][y] = new MemoryField(0, null, null, -1);
		if (date < getDate(x, y))
			return(false);
		else {
			fields[x][y].setFood(food);
			fields[x][y].setAgent(agent);
			fields[x][y].setAgentName(agentName);
			fields[x][y].setDate(date);
			return(true);
		}
	}
	public int getFreeAgentsInRange(int range){
		int x, y;
		int temp = 0;
		MemoryField f;
		for (y=-range; y<=range; y++){
			for (x=-range; x<=range; x++){
				if (x!=0 || y!=0){ //nicht das eigene Feld
					f = fields[World.correctX(agent.getX()+x)][World.correctY(agent.getY()+y)];
					if (f.getAgent()!=null && !f.getAgent().hasCoordinator() && !f.getAgent().isCoordinator()){
						temp++;
					}
				}
			}
		}
		return(temp);
	}
	public int getCoordinatorsInRange(int range){
		int x, y;
		int temp = 0;
		MemoryField f;
		for (y=-range; y<=range; y++){
			for (x=-range; x<=range; x++){
				if (x!=0 || y!=0){ //nicht das eigene Feld				
					f = fields[World.correctX(agent.getX()+x)][World.correctY(agent.getY()+y)];
					if (f.getAgent()!=null && f.getAgent().isCoordinator()){
						temp++;
					}
				}
			}
		}
		return(temp);
	}
	public void merge(Memory memo){
		int i, j;
		int X = World.getInt("World.fieldsX");
		int Y = World.getInt("World.fieldsY");
	
		for (j=0; j<Y; j++){
			for (i=0; i<X; i++){
				if (memo.fields[i][j] != null){
					if (fields[i][j] == null){
						fields[i][j] = new MemoryField(0, null, null, -1);
					}
					fields[i][j].update(memo.fields[i][j]);
				}
			}
		}
	}
	
	public float evaluateTarget(Point p){
		float quali = 0f;
		float quant = 0f;
		float tempQuali, tempQuant;
		int i, j;

		if (hasAgent(p.x, p.y)){
			return(0f);
		}
		else{ 
			for (j=-1; j<=1; j++){
				for (i=-1; i<=1; i++){
					if (fields[World.correctX(p.x+i)][World.correctY(p.y+j)] != null){
						tempQuali = evaluateQualiSubTarget(p.x+i, p.y+j);
						tempQuant = fields[World.correctX(p.x+i)][World.correctY(p.y+j)].getFood() / World.getFloat("World.averageFoodOnField");
						quali = quali + tempQuali;
						quant = quant + tempQuant;
						if (i==0 && j==0){// Zielfeld zhlt doppelt
							quali = quali + tempQuali;
							quant = quant + tempQuant;
						}
					}
				}
			}
			return(quali * quant);
		}
	}
		
	private float evaluateQualiSubTarget(int x, int y){
		MemoryField mf = fields[World.correctX(x)][World.correctY(y)];
		
		if(mf==null || (mf != null && mf.getDate() > agent.getMemoryMaxAge())){
			return(0f); //unbekanntes Feld
		}
		else{
			if(mf.getFood()>0 && mf.getAgentName()==null){
				return(2f); //Nahrung und kein Agent
			}
			else if(mf.getFood()>0  && mf.getAgentName()!=null){
				return(1f); //Nahrung und Agent
			}
			else if(mf.getFood()==0  && mf.getAgentName()==null){
				return(0f); // leeres Feld
			}
			else if(mf.getFood()==0  && mf.getAgentName()!=null){
				return(-2f); // keine Nahrung aber Agent
			}
			else{ // hab ich was bersehen?
				return(0f);
			}
		}
	}
				
	public boolean isKnown(int x, int y){
		if (x < 0) 
			x += World.getInt("World.fieldsX");
		if (x >= World.getInt("World.fieldsX")) 
			x -= World.getInt("World.fieldsX");
		if (y < 0) 
			y += World.getInt("World.fieldsY");
		if (y >= World.getInt("World.fieldsY")) 
			y -= World.getInt("World.fieldsY");
			
		if (fields[x][y] != null)
			return(World.round - fields[x][y].getDate() < agent.getMemoryMaxAge());
		else
			return(false);
	}
	
	public float getFood(int x, int y){
		if (x < 0) 
			x += World.getInt("World.fieldsX");
		if (x >= World.getInt("World.fieldsX")) 
			x -= World.getInt("World.fieldsX");
		if (y < 0) 
			y += World.getInt("World.fieldsY");
		if (y >= World.getInt("World.fieldsY")) 
			y -= World.getInt("World.fieldsY");
		if (fields[x][y] == null)
			return(-1);
		else
			return(fields[x][y].getFood());
	}

	public Color getColor(int x, int y){
		if (isKnown(x, y)){
			float maxFood = 2 * World.getFloat("World.averageFoodOnField");
			float value = Math.max(0f, (fields[x][y].getFood()));	// nicht kleiner 0
			value = Math.min(maxFood, value);		// nicht groesser maxFood
			return(new Color(1f - value / maxFood, 1f, 1f - value / maxFood));
		}
		else
			return(Color.black);
	}

	public String getAgentName(int x, int y){
		if (x < 0) 
			x += World.getInt("World.fieldsX");
		if (x >= World.getInt("World.fieldsX")) 
			x -= World.getInt("World.fieldsX");
		if (y < 0) 
			y += World.getInt("World.fieldsY");
		if (y >= World.getInt("World.fieldsY")) 
			y -= World.getInt("World.fieldsY");
		if (fields[x][y] == null)
			return(null);
		else
			return(fields[x][y].getAgentName());
	}
	public boolean hasAgent(int x, int y){
		if (fields[x][y] != null && fields[x][y].getAgentName() != null 
				&& World.round - fields[x][y].getDate() < agent.getMemoryMaxAge())
			return(true);
		else
			return(false);
	}
		
	public int getDate(int x, int y){
		if (x < 0) 
			x += World.getInt("World.fieldsX");
		if (x >= World.getInt("World.fieldsX")) 
			x -= World.getInt("World.fieldsX");
		if (y < 0) 
			y += World.getInt("World.fieldsY");
		if (y >= World.getInt("World.fieldsY")) 
			y -= World.getInt("World.fieldsY");
		if (fields[x][y] == null)
			return(-1);
		else
			return(fields[x][y].getDate());
	}
	public Agent getAgent(){
		return(agent);
	}
}