import java.io.*;
import java.awt.*;
import java.util.*;

public class Satisfaction implements Serializable{
	public static final int SURVIVAL = 0;
	public static final int WEALTH = 1;
	public static final int CURIOSITY = 2;
	public static final int BREEDING = 3;
	public static final int INFLUENCE = 4;
	
	public static final int GATHER = 0;
	public static final int MOVE = 1;
	public static final int BREED = 2;
	public static final int START_COORDINATING = 3;
	public static final int END_COORDINATING = 4;
	public static final int SUBORDINATE = 5;
	public static final int UNSUBORDINATE = 6;
	public static final int REST = 7;

	private float defaultSurvival;
	private float defaultWealth;
	private float defaultCuriosity;
	private float defaultBreeding;
	private float defaultInfluence;
		
	private float[] weight = new float[5];					//fr jedes Bedrfnis 1
	private float[] estimatedSatis = new float[8]; 	//fr jede Ttigkeit 1
	private float[] satis = new float[5];						//fr jedes Bedrfnis 1
	private Agent agent;
	private Point whereToMove;
	
	public Satisfaction(Agent a){
		agent = a;
	}
	
	public void setWeights(float survivalW, float wealthW, float curiosityW, float breedingW, float influenceW){
		weight[0] = survivalW;
		weight[1] = wealthW;
		weight[2] = curiosityW;
		weight[3] = breedingW;
		weight[4] = influenceW;
	}
	
	//Auswahl der Aktion aufgrund der geschtzten Befriedigung
	public int getAction(){
		//ersmal ne Kopie machen
		float[] estSat = new float[8];
		for (int i=0; i<estSat.length; i++){
			estSat[i] = estimatedSatis[i];
		}
		//zuerst kleinste satis >0 suchen 
		float tempMin = 1;
		for (int i=0; i<estSat.length; i++){
			if (estSat[i] > 0 && estSat[i] < tempMin){
				tempMin = estSat[i];
			}
		}
		//diesen Wert dann von allen satis >0 abziehen
		for (int i=0; i<estSat.length; i++){
			if (estSat[i] > 0){
				estSat[i] = estSat[i] - tempMin;
			}
		}
		float sum = 0f;
		for (int i=0; i<estSat.length; i++){
			sum = sum + estSat[i];
		}
		if (sum == 0f){ // falls nur eine Aktion mglich ist ('rest' geht immer)...
			return(REST);
		}
		//float rnd = (float)Math.random() * sum;
		float rnd = World.random.nextFloat() * sum;
		sum = 0;
		for (int i=0; i<estSat.length; i++){
			sum = sum + estSat[i];
			if (rnd <= sum)
				return(i);
		}
		System.out.println("in Satisfaction.getAction(): Zufallsauswahl hat versagt");
		return(0);
	}

	public void calcEstimatedSatis(){
		//berechnet fr alle Aktionen die estSat und speichert sie im Array  estimatedSatis
		//zur Verwendung in getAction
		float[] baseValues;
		float[] foodBasedSatis;
		//zuerst werden fr bestimmte Bedrfnisse defaultWerte berechnet:
		//Grundwerte werden fr das aktuelle Feld berechnet
		baseValues = baseValuesForField(agent.getX(), agent.getY(), agent.hasCoordinator(), 0);
		foodBasedSatis = foodBasedSatis(agent.getFood(), baseValues[0], agent.isCoordinator());

		defaultSurvival = foodBasedSatis[0];
		defaultWealth = foodBasedSatis[1];
		defaultCuriosity = baseValues[1];
		if (agent.getAge() < agent.getPubertyAge()){
			defaultBreeding = 1;
		}
		else if (agent.getAge() == agent.getPubertyAge()){
			defaultBreeding = 0;
		}
		else if (agent.getNumOfChildren() >= World.getInt("Agent.maxNumOfChildren")){
			defaultBreeding = 1;
		}
		else{
			defaultBreeding = satis[BREEDING] * World.getFloat("Agent.birthRecreationFactor");
		}
		defaultInfluence = saturation(agent.getNumOfSubordinates(), World.getInt("Agent.averageNumOfSubs"));
		
		//die einzelnen Funktionen werden aufgerufen und die Returnwerte entsprechend gespeichert
		estimatedSatis[GATHER] = estimatedSatisGather();
		estimatedSatis[MOVE] = estimatedSatisMove();
		estimatedSatis[BREED] = estimatedSatisBreed();
		estimatedSatis[START_COORDINATING] = estimatedSatisStartCoordinating();
		estimatedSatis[END_COORDINATING] = estimatedSatisEndCoordinating();
		estimatedSatis[SUBORDINATE] = estimatedSatisSubordinate();
		estimatedSatis[UNSUBORDINATE] = estimatedSatisUnsubordinate();
		estimatedSatis[REST] = estimatedSatisRest();
		if (World.debug){
			System.out.print("in Satisfaction.calcEstimatedSatis:");
			System.out.println(agent.getName());
			System.out.print("Gather: ");
			System.out.println(estimatedSatis[GATHER]);
			System.out.print("Move: ");
			System.out.println(estimatedSatis[MOVE]);
			System.out.print("Breed: ");
			System.out.println(estimatedSatis[BREED]);
			System.out.print("StartCoordinating: ");
			System.out.println(estimatedSatis[START_COORDINATING]);
			System.out.print("EndCoordinating: ");
			System.out.println(estimatedSatis[END_COORDINATING]);
			System.out.print("Subordinate: ");
			System.out.println(estimatedSatis[SUBORDINATE]);
			System.out.print("Unsubordinate: ");
			System.out.println(estimatedSatis[UNSUBORDINATE]);
			System.out.print("Rest: ");
			System.out.println(estimatedSatis[REST]);
		}
	}
	
	//[0] food; [1] curiosity
	//berechnet die Grundwerte, die durch die Umgebung fr Nahrung und Neugier bestimmt werden
	//dabei wird der Wert duch Koordinierungssteuer und Sammeln korrigiert 
	private float[] baseValuesForField(int x, int y, boolean coordinated, float gatherAmount){
		float[] result = new float[2];
		float baseValueFood = 0f;
		float baseValueUnknown = 0f;		
		Vector v = null;
		Point p = null;
		int fieldNum;
		float avFood = World.getFloat("World.averageFoodOnField");
		Memory m = agent.getMemory();
		float sumFood;
		float sumUnknown;
		float tempFood;
		
		for (int i=agent.getPlanningRange(); i>=0; i--){
			v = World.getCoordinatesInDistance(x, y, i);
			fieldNum = v.size();
			sumFood = 0f;
			sumUnknown = 0f;
			while (v.size()>0){
				p = (Point)v.remove(v.size()-1);
				if (m.isKnown(p.x, p.y)){
					tempFood = m.getFood(p.x, p.y);
					if (i==0) {	//das Feld auf dem der Agent selber steht
						tempFood = tempFood - gatherAmount;
					}
					else if (m.hasAgent(p.x, p.y)){
						tempFood = tempFood / 2f;
					}
					sumFood = sumFood + tempFood;
				}
				else{
					sumUnknown++;
				}
				
			}
			sumFood = sumFood / avFood;
			sumFood = sumFood / fieldNum;
			baseValueFood = baseValueFood / 2 + sumFood;
			sumUnknown = sumUnknown / (float)fieldNum;
			baseValueUnknown = baseValueUnknown / 2 + sumUnknown;
		}
		if (coordinated){
			baseValueFood = baseValueFood * (1 - World.getFloat("Agent.coordinatorContribution"));
		}
		baseValueFood = baseValueFood / 2;
		baseValueUnknown = baseValueUnknown / 2;
		if (baseValueFood > 1){
			baseValueFood = 1f;
		}
		result[0] = baseValueFood;
		result[1] = baseValueUnknown;
		if (World.debug){
			System.out.println("baseValueFood: " + result[0]);
			System.out.println("baseValueUnknown: " + result[1]);
		}
		return(result);
	}

	

	/**
	 * Berechnet aus den bergebenen Werten die Zufriedenheit fr
	 * survival und wealth.
	 * @param   food der Nahrungsvorrat des Agenten.
	 * @param   baseValueFood der Grundwert, berechnet aus der Nahrung in der Umgebung.
	 * @param   coordinating ist der Agent ein Koordinator.
	 * @return Array aus zwei float Werten {survival, wealth}.
	**/
	public float[] foodBasedSatis(float food, float baseValueFood, boolean coordinating){
		float[] result = new float[2];//[0] survival   [1] wealth
		float temp;
		//survival = 90% aus konkretem food-Bestand + 10% aus baseValue 
		if (coordinating){
			temp = food / (agent.getEatFoodPR() 
											* World.getFloat("Agent.coordinatorsFoodPRFactor") 
											* World.getInt("Agent.roundsToSurvive"));
		}
		else{
			temp = food / (agent.getEatFoodPR() * World.getInt("Agent.roundsToSurvive"));
		}
		temp = Math.min(1f, temp);
		result[0] = 0.9f * temp + 0.1f * baseValueFood;

		//wealth = 90% food + 10% Umgebung
		result[1] = 0.9f * saturation(food, World.getFloat("Agent.averageWealth")) 
							+ 0.1f * baseValueFood;
		return(result);
	}
	
	public float estimatedSatisMove(){
		float bestSatis = 0;
		Point bestField = null;
		int minimalTargetDistance = (int)Integer.MAX_VALUE;
		int d;
		Vector v;
		Point target = agent.getTarget();
		Point p;
		float[] baseValues, foodBasedSatis;
		float tempSatis;
		
		if (agent.canMove){
			v = World.getCoordinatesInDistance(agent.getX(), agent.getY(),1);
			for (int i = 0; i<v.size(); i++){ //Schleife ber alle Nachbarfelder
				p = (Point)v.elementAt(i);
				if (agent.getMemory().hasAgent(p.x, p.y)){ //Feld ist besetzt
					continue;//weiter mit der for-Schleife
				}
				if (target != null){
					d = World.distance(p.x, p.y, target.x, target.y);
					if (d > minimalTargetDistance){ //fhrt nicht direkt zum Target
						continue; // weiter mit der for-Schliefe
					}
					minimalTargetDistance = d;
				}
				baseValues = baseValuesForField(p.x, p.y, agent.hasCoordinator(), 0);
				foodBasedSatis = foodBasedSatis(agent.getFood(), baseValues[0], agent.isCoordinator());
				tempSatis = combineSatisfaction(foodBasedSatis[0], foodBasedSatis[1], baseValues[1], defaultBreeding, defaultInfluence); 
				if (tempSatis > bestSatis){
					bestSatis = tempSatis;
					bestField = p;
				}
			}
			whereToMove = bestField;
			return(bestSatis);
		}
		else{
			return(0);
		}
	}
	public float estimatedSatisGather(){
		Memory memory = agent.getMemory();
		float amount; 
		float[] foodBasedSatis;
		float[] baseValues;
		if (agent.canGather){
			float satis;
			amount = Math.min(agent.getGatherAmount(), memory.getFood(agent.getX(), agent.getY())-agent.getGatherRest());
			baseValues = baseValuesForField(agent.getX(), agent.getY(), agent.hasCoordinator(), amount);
			foodBasedSatis = foodBasedSatis(agent.getFood() + amount, baseValues[0], agent.isCoordinator());
			satis = combineSatisfaction(foodBasedSatis[0], foodBasedSatis[1], baseValues[1], defaultBreeding, defaultInfluence);
			return(satis);
		}
		else{
			return(0f);
		}
	}
	public float estimatedSatisBreed(){
		float[] baseValues;
		float[] foodBasedSatis;
		float food;
		float sat;
		if (agent.canBreed){
			baseValues = baseValuesForField(agent.getX(), agent.getY(), agent.hasCoordinator(), 0);
			food = agent.getFood() - agent.getChildFood();
			foodBasedSatis = foodBasedSatis(food, baseValues[0], agent.isCoordinator());
			sat = combineSatisfaction(foodBasedSatis[0], foodBasedSatis[1], baseValues[1], 1, defaultInfluence);
			return(sat);
		}
		else{
			return(0);
		}
	}
	public float estimatedSatisStartCoordinating(){
		Memory memory = agent.getMemory();
		float[] baseValues;
		float[] foodBasedSatis;
		float sat;
		int freeAgents = 0;
		int coordinators = 0;
		float survival, wealth, breeding, curiosity, influence;
		float factor;
		
		if (agent.canStartCoordinating){
			baseValues = baseValuesForField(agent.getX(), agent.getY(), agent.hasCoordinator(), 0);
			foodBasedSatis = foodBasedSatis(agent.getFood(), baseValues[0], true);
			survival = foodBasedSatis[0];
			//Malus auf wealth wg erhhtem Grundumsatz z.B. *0.9 (survival schon bei foodBasedSat bercksichtigt)
			wealth = foodBasedSatis[1] * (1-World.getFloat("Agent.coordinatorMalus"));
			curiosity = baseValues[1];
			breeding = defaultBreeding;
			freeAgents = memory.getFreeAgentsInRange(agent.getViewingRange());
			if (World.debug){
				System.out.print("FreeAgentsInRange: ");
				System.out.println(freeAgents);
			}
			coordinators = memory.getCoordinatorsInRange(agent.getViewingRange()) + 1;
			influence = saturation((float)freeAgents / (float)coordinators, World.getFloat("Agent.averageNumOfSubs"));
			//Bonus
			factor = saturation((float)freeAgents / (float)coordinators, 
													World.getFloat("Agent.averageNumOfSubs"),
													World.getFloat("Agent.coordinatorBonusStrength"));
			survival = survival + (1-survival) * factor;
			wealth = wealth + (1-wealth) * factor;
			curiosity = curiosity + (1-curiosity) * factor;
			sat = combineSatisfaction(survival, wealth, curiosity, breeding, influence);
			return(sat);
		}
		else{
			return(0);
		}
	}
	public float estimatedSatisEndCoordinating(){
		Memory memory = agent.getMemory();
		float[] baseValues;
		float[] foodBasedSatis;
		float sat;
		int subs = agent.getNumOfSubordinates();
		float survival, wealth, breeding, curiosity, influence;
		float factor;

		if (agent.canEndCoordinating){
			baseValues = baseValuesForField(agent.getX(), agent.getY(), agent.hasCoordinator(), 0);
			foodBasedSatis = foodBasedSatis(agent.getFood(), baseValues[0], false);
			survival = foodBasedSatis[0];
			wealth = foodBasedSatis[1]; //?? + (1-foodBasedSatis[1])*World.getFloat("Agent.coordinatorBonus");
			curiosity = baseValues[1];
			breeding = defaultBreeding;
			influence = 0;
			factor = saturation(subs, World.getFloat("Agent.averageNumOfSubs"), World.getFloat("Agent.endCoordinatingMalus"));
			survival = survival * (1-factor);
			wealth = wealth * (1-factor);
			curiosity = curiosity * (1-factor);
			sat = combineSatisfaction(survival, wealth, curiosity, breeding, influence);
			//Schonfrist
			if (agent.startCoordinatingDate > -1){
				sat = sat * Math.min(1, (float)(World.round - agent.startCoordinatingDate) / (float)(World.getInt("Agent.probationTime")));
			}
			return(sat);
		}
		else{
			return(0);
		}
	}
	
	public float estimatedSatisSubordinate(){
		float[] baseValues;
		float[] foodBasedSatis;
		float sat;
		float survival, wealth;
		float value;
		float factor;
		if (agent.canSubordinate){
			baseValues = baseValuesForField(agent.getX(), agent.getY(), true, 0);
			foodBasedSatis = foodBasedSatis(agent.getFood(), baseValues[0], agent.isCoordinator());
			survival = foodBasedSatis[0];
			wealth = foodBasedSatis[1];
			//ToDo Wert eines Koordinators ausprogrammieren
			value = agent.getBestCoordinatorValue();
			factor = saturation(Math.abs(value), 
				                  World.getFloat("Agent.averageWealth"),
				                  World.getFloat("Agent.subordinationCoordinatorsValueInfluence"));
			if (value > 0){
				survival = survival + (1-survival) * factor;
				wealth = wealth + (1-wealth) * factor;
			}
			else{
				survival = survival * (1-factor);
				wealth = wealth * (1-factor);
			}
			sat = combineSatisfaction(survival, wealth, defaultCuriosity, defaultBreeding, defaultInfluence);
			return(sat);
		}
		else{
			return(0);
		}
	}
	public float estimatedSatisUnsubordinate(){
		float[] baseValues;
		float[] foodBasedSatis;
		float sat;
		float survival, wealth;
		float value;
		float factor;
		if (agent.canUnsubordinate){
			baseValues = baseValuesForField(agent.getX(), agent.getY(), false, 0);
			foodBasedSatis = foodBasedSatis(agent.getFood(), baseValues[0], agent.isCoordinator());
			survival = foodBasedSatis[0];
			wealth = foodBasedSatis[1];
			value = agent.getCoordinatorValue();			
			factor = saturation(Math.abs(value), 
				                  World.getFloat("Agent.averageWealth"),
				                  World.getFloat("Agent.unsubordinationCoordinatorsValueInfluence"));
			if (value > 0){ //guter Koordinator wird verlassen
				survival = survival * (1-factor);
				wealth = wealth * (1-factor);
			}
			else{
				survival = survival + (1-survival) * factor;
				wealth = wealth + (1-wealth) * factor;
			}
			sat = combineSatisfaction(survival, wealth, defaultCuriosity, defaultBreeding, defaultInfluence);
			if (agent.subordinationDate > -1){
				sat = sat * Math.min(1, (float)(World.round - agent.subordinationDate) / (float)(World.getInt("Agent.probationTime")));
			}
			return(sat);
		}
		else{
			return(0);
		}
	}
	public float estimatedSatisRest(){
		float satisfaction = combineSatisfaction(defaultSurvival, defaultWealth, defaultCuriosity, defaultBreeding, defaultInfluence); 
		return(satisfaction);
	}

	public float saturation(float x){
		return(saturation(x, 1, 1));
	}
	public float saturation(float x, float norm){
		return(saturation(x, norm, 1));
	}
	//saturation(norm, norm, 1) = 0,5
	public float saturation(float x, float norm, float intensity){
		if (World.debug){
			System.out.println("Saturation:");
			System.out.print("x: "+x+"; norm: "+norm+"; intensity: "+intensity);
		}
		if (x<0 || norm<=0 || intensity<0){
			throw(new IllegalArgumentException("unzulssiger Parameter in Satisfaction.saturation\n x="+x+"; norm="+norm+"; intens="+intensity));
		}
		if (intensity <= 1){
			if (World.debug){
				System.out.println("; result"+(float)(Math.atan(x/norm)/(Math.PI/2)) * intensity);
			}
			return((float)(Math.atan(x/norm)/(Math.PI/2)) * intensity);
		}
		else{
			if (World.debug){
				System.out.println("; result"+(float)Math.atan((x/norm)*intensity)/(float)(Math.PI/2));
			}
			return((float)Math.atan((x/norm)*intensity)/(float)(Math.PI/2));
		}
	}
	public float combineSatisfaction(float survival, float wealth, float curiosity, float breeding, float influence){
		return(survival * weight[SURVIVAL] 
						+ wealth * weight[WEALTH] 
						+ curiosity * weight[CURIOSITY] 
						+ breeding * weight[BREEDING]
						+ influence * weight[INFLUENCE]);
	}
	
	public float getSatis(int i){
		return(satis[i]);
	}
	public float getEstSatis(int i){
		return(estimatedSatis[i]);
	}

	public void setSatis(int i, float value){
		satis[i] = value;
	}
	public Point getWhereToMove(){
		return(whereToMove);
	}
}