import java.util.*;
import java.io.*;
import java.awt.*;

/**
 * Hier wird die Simulation angetrieben.
**/
public class Scheduler extends Thread{
	private Vector agents;
	private WorldView worldView;
	public static boolean run = false;
	public static boolean stillRunning = false; //true, wenn die aktuelle Runde noch beendet wird
	private boolean recordMode = false;
	private Vector saveChanges;
	private ObjectOutputStream recordOutStream;
	
	/**
	 * Der Konstruktor...
	**/
	public Scheduler(){
		agents = World.getAgentsClone();
	}
	
	/**
	 * Setzt einen Verweis auf WorldView
	 * @param vw der Verweis auf WorldView.
	**/
	public void setWorldView(WorldView vw){
		worldView = vw;
	}
	
	/**
	 * Fgt einen Agenten in die Liste der zu bearbeitenden Agenten ein.
	 * Und fgt ihn ins Menu "Agents" ein.
	 * @param a der Agent.
	**/
	public void addAgent(Agent a){
		agents.addElement(a);
	}
	
	/**
	 * Lscht einen Agenten aus der Liste der zu bearbeitenden Agenten.
	 * @param a der Agent.
	**/	
	public void removeAgent(Agent a){
		agents.remove(a);
	}
	
	public void setRecordMode(boolean flag){
		recordMode = flag;
	}
	
	public boolean getRecordMode(boolean flag){
		return(recordMode);
	}
	
	public void setRecordStream(ObjectOutputStream ros){
		recordOutStream = ros;
	}
	
	public ObjectOutputStream getRecordStream(){
		return(recordOutStream);
	}

	/**
	 * Fhrt einen Simulationsschritt durch (eine Runde).
	**/
	public void step(){	
		int x, y;
		boolean changed;
		try{
			File monitorFile = new File("simulationOk");
			monitorFile.createNewFile();
		}catch (Exception e){}
		
		Stat.newRound();
		World.round++;
		if (World.maxRound > 0 && World.round > World.maxRound){
			File okFile = new File("ok");
			try{
				okFile.delete();
				okFile.createNewFile();
			}
			catch(Exception e){}
			System.exit(0);
		}
		worldView.updateRound();

		if (recordMode){
			saveChanges = new Vector();
		}
		else{
			saveChanges = null;
		}
		
		//Spreading
		float prob = World.getFloat("Spread.probability");
		float perc = World.getFloat("Spread.percentage");
		float spreadAmount;
		Field tempField;
		Field targetField;
		Vector tempV;
		Point tempP;
		if (prob>0 && perc>0){
			for (y=0; y<World.getInt("World.fieldsY"); y++){
				for (x=0; x<World.getInt("World.fieldsX"); x++){
					tempField = World.getField(x, y);
					if (prob >= World.random.nextFloat() && tempField.getFertility()){
						spreadAmount = (tempField.getFood() - GrowFunction.getMin()) * perc;
						if (spreadAmount <= 0){
							spreadAmount = 0;
						}
						else{
							tempField.setFood(tempField.getFood()-spreadAmount);
							tempV = World.getRandomizedCoordinatesInDistance(x, y, 1);
							tempP = (Point)(tempV.get(0));
							targetField = World.getField(tempP.x, tempP.y);
							if (targetField.getFertility()){
								targetField.setFood(targetField.getFood() + spreadAmount);
								worldView.getTorusView().paintField(tempP.x, tempP.y, saveChanges);
							}
							worldView.getTorusView().paintField(x, y, saveChanges);
						}
					}
				}
			}
		}
		
		//Growing
		for (y=0; y<World.getInt("World.fieldsY"); y++){
			for (x=0; x<World.getInt("World.fieldsX"); x++){
				changed = World.getField(x, y).grow();
				if (changed){
					worldView.getTorusView().paintField(x, y, saveChanges);
				}
				if (World.getField(x,y).getFood() >= GrowFunction.getMin()){
					Stat.addFoodField(World.getField(x,y).getFood());
				}
			}
		}
		//jetzt die Agenten
		Vector agentsToGo = World.getAgentsClone();
		Collections.shuffle(agentsToGo, World.random);
		while (agentsToGo.size() > 0){
			Agent a =	((Agent)agentsToGo.remove(agentsToGo.size()-1));
			int oldX = a.getX();
			int oldY = a.getY();
			if (a.isAlive()){
				a.go();
				AgentTree.update(a);
				Stat.addAgent(a);
			}
			if (a.isVisible()){
				worldView.getTorusView().paintField(oldX, oldY, saveChanges);
				worldView.getTorusView().paintField(a.getX(), a.getY(), saveChanges);
			}
			else{		
				a.removeMemoryMonitor();
				a.removeAgentMonitor();
				a.removeAgent();
				if (World.getBoolean("World.funeralService")){
					World.removeAgent(a);
				}
				if (World.getField(a.getX(), a.getY()).getAgent() == a){
					World.getField(a.getX(), a.getY()).setAgent(null);
					worldView.getTorusView().paintField(a.getX(), a.getY(), saveChanges);
				}
				else{
					//dann hat sich schon der nchste Agent auf das "freie" Feld gesetzt
				}
			}
		}
		
		Stat.endRound();
		if (recordMode){
			try{
	 			recordOutStream.writeObject(saveChanges);
			}
			catch (Exception e){
				System.out.println("in Scheduler.step: " + e);
			}	
		}
		if (!run){
			World.setEditMode(true);
		}
	}
	
	/**
	 * Lsst die Simulation laufen bis der Modus auf 'Edit' gendert wird.
	 * Dann wird noch die laufende Runde abgeschlossen.
	**/
	public void run(){
		int asi; //autoSaveInterval
		String filename;
		while (true){
			if (run){
				step();
				asi = World.getInt("World.autoSaveInterval");
				if ( asi > 0 && World.round % asi == 0){ //dann AutoSave aktivieren
					filename = "000000" + World.round;
					filename = filename.substring(filename.length() - 6);
					filename = "autosave" + filename;
					worldView.saveWorld(filename);
				}
				try{
					Thread.sleep(World.getInt("World.slowDown"));
				}
				catch (Exception e){}
			} 
			else{
				try{
					Thread.sleep(10);
				}
				catch (Exception e){}
			}
		}
	}
}