import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Dient der Anzeige der Felder. Ist sein eigener MouseListener.
 */
public class TorusView extends Panel implements MouseListener, MouseMotionListener {
	
	private boolean paintFoodOn = false;
	private Memory memory = null; //falls != null so dient es der Anzeige eines Memorys
		
	/**
	 * Schafft ein neues Panel fuer die Anzeige eines Memorys.
	**/
	public TorusView(Memory m){
		this();
		memory = m;
	}
	
	/**
	 * Schafft ein neues Panel und setzt den Layoutmanager (derzeit auf BorderLayout).
	**/
	public TorusView(){
		this.setLayout(new BorderLayout());
		this.setBackground(Color.lightGray);
		this.addMouseListener(this);
		this.addMouseMotionListener(this);		
	}

	/**
	 * Wird vom System zum Neuzeichnen aufgerufen.
	**/
	public void paint(Graphics g){
		if (World.silent) return;
		if (memory == null)
			paintWorld(g);
		else{
			paintMemory(g);
		}
	}
	
	public void paintWorld(Graphics g){
		int maxX = World.getInt("World.fieldsX"); // Anzahl der Felder in X-Richtung
		int maxY = World.getInt("World.fieldsY"); // Anzahl der Felder in Y-Richtung
		int ix, iy;
		int fWidth  = (getWidth()-getInsets().left-getInsets().right-2) / (maxX) - 1;
		int fHeight = (getHeight()-getInsets().top-getInsets().bottom-2) / (maxY) - 1;
		Field tempF = null;
		
		for (iy=0; iy<maxY; iy++){
			for (ix=0; ix<maxX; ix++){
				tempF = World.getField(ix, iy);
				g.setColor(tempF.getColor());
				g.fillRect(getInsets().left+2+ix*(fWidth+1), getInsets().top+2+iy*(fHeight+1), fWidth, fHeight);
				if (tempF.hasAgent()) {
					g.setColor(tempF.getAgent().getColor());
					int xStart = getInsets().left+3+ix*(fWidth+1);
					int yStart = getInsets().top+3+iy*(fHeight+1);
					if (tempF.getAgent().isCoordinator()){
						Polygon triangle = World.getCoordinatorPolygon(xStart, yStart, fWidth-2, fHeight-2);
						g.fillPolygon(triangle);
					}
					else if (tempF.getAgent().hasCoordinator()){
						Polygon triangle = World.getSubordinatePolygon(xStart, yStart, fWidth-2, fHeight-2);
						g.fillPolygon(triangle);
					}
					else{
						g.fillOval(xStart, yStart, fWidth-2, fHeight-2);
					}
				}
			}
		}
		if (World.getEditMode()){
			Point p = World.getCursorPosition();
			g.setColor(Color.red);
			g.drawRect(p.x*(fWidth+1)+1, p.y*(fHeight+1)+1, fWidth+1, fHeight+1);
		}
	}

	public void paintMemory(Graphics g){
		int maxX = World.getInt("World.fieldsX"); // Anzahl der Felder in X-Richtung
		int maxY = World.getInt("World.fieldsY"); // Anzahl der Felder in Y-Richtung
		int ix, iy;
		int fWidth  = (getWidth()-getInsets().left-getInsets().right-2) / (maxX) - 1;
		int fHeight = (getHeight()-getInsets().top-getInsets().bottom-2) / (maxY) - 1;
				
		for (iy=0; iy<maxY; iy++){
			for (ix=0; ix<maxX; ix++){
				g.setColor(memory.getColor(ix, iy));
				g.fillRect(getInsets().left+2+ix*(fWidth+1), getInsets().top+2+iy*(fHeight+1), fWidth, fHeight);
				if (memory.hasAgent(ix, iy)) {
					if (memory.getAgentName(ix, iy).equals(memory.getAgent().getName())){
						g.setColor(Color.orange);
					}
					else{
						g.setColor(Color.blue);
					}
					g.fillOval(getInsets().left+3+ix*(fWidth+1), getInsets().top+3+iy*(fHeight+1), fWidth-2, fHeight-2);
				}
			}
		}
	}

	
	/**
	 * Zeichnet ein einzelnes Feld.
	 * @param xF x-Koordinate des Feldes
	 * @param yF y-Koordinate des Feldes
	**/
	public void paintField(int xF, int yF, Vector saveChanges){
		Graphics g = getGraphics();
		int maxX = World.getInt("World.fieldsX"); // Anzahl der Felder in X-Richtung
		int maxY = World.getInt("World.fieldsY"); // Anzahl der Felder in Y-Richtung
		int fWidth  = (getWidth()-getInsets().left-getInsets().right-2) / (maxX) - 1;
		int fHeight = (getHeight()-getInsets().top-getInsets().bottom-2) / (maxY) - 1;
		Color fieldColor;
		Color agentColor = null;
		boolean hasAgent;

		
		fieldColor = World.getField(xF, yF).getColor();
		g.setColor(fieldColor);
		g.fillRect(getInsets().left+2+xF*(fWidth+1), getInsets().top+2+yF*(fHeight+1), fWidth, fHeight);
		if (World.getField(xF, yF).hasAgent()) {
			int xStart = getInsets().left+3+xF*(fWidth+1);
			int yStart = getInsets().top+3+yF*(fHeight+1);
			hasAgent = true;
			agentColor = World.getField(xF, yF).getAgent().getColor();
			g.setColor(agentColor);
			if (World.getField(xF, yF).getAgent().isCoordinator()){
				Polygon triangle = World.getCoordinatorPolygon(xStart, yStart, fWidth-2, fHeight-2);
				g.fillPolygon(triangle);
			}
			else if (World.getField(xF, yF).getAgent().hasCoordinator()){
				Polygon triangle = World.getSubordinatePolygon(xStart, yStart, fWidth-2, fHeight-2);
				g.fillPolygon(triangle);
			}
			else{
				g.fillOval(getInsets().left+3+xF*(fWidth+1), getInsets().top+3+yF*(fHeight+1), fWidth-2, fHeight-2);
			}
		}
		else{
			hasAgent = false;
		}
	}
	
	public void showCursor(){
		int maxX = World.getInt("World.fieldsX"); // Anzahl der Felder in X-Richtung
		int maxY = World.getInt("World.fieldsY"); // Anzahl der Felder in Y-Richtung
		int fWidth  = (getWidth()-getInsets().left-getInsets().right-2) / (maxX) - 1;
		int fHeight = (getHeight()-getInsets().top-getInsets().bottom-2) / (maxY) - 1;
		
		Graphics g = getGraphics();
		Point p = World.getCursorPosition();
		g.setColor(Color.red);
		g.drawRect(p.x*(fWidth+1)+1, p.y*(fHeight+1)+1, fWidth+1, fHeight+1);
	}
	
	public void hideCursor(){
		int maxX = World.getInt("World.fieldsX"); // Anzahl der Felder in X-Richtung
		int maxY = World.getInt("World.fieldsY"); // Anzahl der Felder in Y-Richtung
		int fWidth  = (getWidth()-getInsets().left-getInsets().right-2) / (maxX) - 1;
		int fHeight = (getHeight()-getInsets().top-getInsets().bottom-2) / (maxY) - 1;
		
		Graphics g = getGraphics();
		Point p = World.getCursorPosition();
		g.setColor(Color.lightGray);
		g.drawRect(p.x*(fWidth+1)+1, p.y*(fHeight+1)+1, fWidth+1, fHeight+1);
	}
	
	public void paintFoodOn(){
		paintFoodOn = true;
	}
	public void paintFoodOff(){
		paintFoodOn = false;
	}

	public void mouseDragged(MouseEvent e){
		if (paintFoodOn && World.getEditMode()){
			int maxX = World.getInt("World.fieldsX"); // Anzahl der Felder in X-Richtung
			int maxY = World.getInt("World.fieldsY"); // Anzahl der Felder in Y-Richtung
			int fWidth  = (getWidth()-getInsets().left-getInsets().right-2) / (maxX) - 1;
			int fHeight = (getHeight()-getInsets().top-getInsets().bottom-2) / (maxY) - 1;
	
			int xPix = e.getPoint().x;
			int yPix = e.getPoint().y;
			int xField = Math.min(xPix / (fWidth+1), World.getInt("World.fieldsX")-1);
			int yField = Math.min(yPix / (fHeight+1), World.getInt("World.fieldsY")-1);
	
			World.getField(xField, yField).setFood(World.getFloat("World.averageFoodOnField"));
			paintField(xField, yField, null);
		}
	}
	
	public void mouseMoved(MouseEvent e){
			int maxX = World.getInt("World.fieldsX"); // Anzahl der Felder in X-Richtung
			int maxY = World.getInt("World.fieldsY"); // Anzahl der Felder in Y-Richtung
			int fWidth  = (getWidth()-getInsets().left-getInsets().right-2) / (maxX) - 1;
			int fHeight = (getHeight()-getInsets().top-getInsets().bottom-2) / (maxY) - 1;
			int xPix = e.getPoint().x;
			int yPix = e.getPoint().y;
			int xField = Math.min(xPix / (fWidth+1), World.getInt("World.fieldsX")-1);
			int yField = Math.min(yPix / (fHeight+1), World.getInt("World.fieldsY")-1);
			StringBuffer text = new StringBuffer("(");
			text.append(xField);
			text.append(", ");
			text.append(yField);
			text.append(")");
			if (World.fields[xField][yField].hasAgent()){
				text.append(" ");
				text.append(World.fields[xField][yField].getAgent().getName());
			}
			text.append(" ");
			if (World.fields[xField][yField].getFertility()){
				text.append("fertile(");
			}
			else{
				text.append("infertile(");
			}
			text.append(World.fields[xField][yField].getFood());
			text.append(")");
			World.setStatusLine(text);
	}

	/**
	 * Wird aktiviert,wenn die Maus im Panel (in einem Feld) geklickt wird.
	 */
	public void mouseClicked(MouseEvent e){
		int maxX = World.getInt("World.fieldsX"); // Anzahl der Felder in X-Richtung
		int maxY = World.getInt("World.fieldsY"); // Anzahl der Felder in Y-Richtung
		int fWidth  = (getWidth()-getInsets().left-getInsets().right-2) / (maxX) - 1;
		int fHeight = (getHeight()-getInsets().top-getInsets().bottom-2) / (maxY) - 1;

		int xPix = e.getPoint().x;
		int yPix = e.getPoint().y;
		int xField = xPix / (fWidth+1);
		int yField = yPix / (fHeight+1);
		
		if (xField >= maxX) xField = maxX-1;
		if (xField < 0) xField = 0;
		if (yField >= maxY) yField = maxY-1;
		if (xField < 0) xField = 0;
		
		if (memory != null){
			return;
		}
		
		if (World.getEditMode()){	//ist im Editier-Modus
			Graphics g = getGraphics();
			Point p = World.getCursorPosition();
			g.setColor(Color.lightGray);
			g.drawRect(p.x*(fWidth+1)+1, p.y*(fHeight+1)+1, fWidth+1, fHeight+1);
			World.setCursorPosition(new Point(xField, yField));
			g.setColor(Color.red);
			g.drawRect(xField*(fWidth+1)+1, yField*(fHeight+1)+1, fWidth+1, fHeight+1);
		}
		if (e.getClickCount() == 1){ //Click
			Agent a = World.getField(xField, yField).getAgent();
			if (a != null){
				a.showMonitors();
			}
		}
		else if (e.getClickCount() == 2){ //DoubleClick
			Agent a = World.getField(xField, yField).getAgent();
			if (a != null){
				a.setAgentMonitor();
				a.setMemoryMonitor();
			}
		}
	}
	public void mousePressed(MouseEvent e){}
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mouseReleased(MouseEvent e){}
}