import java.util.*;
import java.io.*;
import java.awt.*;
import java.applet.*;
import javax.swing.tree.*;
import java.net.*;

/**
 * Die allesumfassende Welt.
**/
public class World implements Serializable{
	
	private static boolean editMode = true;
	private static Point cursorPosition = new Point(0, 0);
	public static WorldView worldView;
	public static int round;
	
	public static int maxRound;
	/** enthlt Pfad und Namen der Konfigurationsdatei (oder null wenn die Defaultdatei: props.config geladen wird). */
	public static String configFile;
	public static String loadFile;
	public static boolean runFile;
	public static boolean sound;
	public static boolean debug;
	public static boolean silent;
	public static String decimal;
	private static Agent agentTreeRoot;
	public static AudioClip birthSound;
	public static AudioClip deathSound;
	public static Random random;

	/**
	 * Alle Properties des Programms; mit Defaultwerten belegt, die
	 * von einer Textdatei ueberschreiben werden koennen.
	**/
	public static Properties props;
	
	/**
	 * Die Liste aller lebenden Agenten.
	**/
	private static Vector agents;
	
	/**
	 * Ein rechteckiges Array mit den Feldern.
	**/
	public static Field[][] fields;
	
	/**
	 * Der Konstruktor setzt u.a. die Defaults und aktuellen
	 * Werte der Properties props
	**/
	public World(String fileName){
		round = 0;
		agents = new Vector();
		setProps(fileName);
		if (getInt("Random.seed") == 0){
			random = new Random();
		}
		else{
			random = new Random(getInt("Random.seed"));
		}
		initAgentTreeRoot();
		fieldsInit();
		agentsInit();
	}
	
	private void fieldsInit(){
		float minFood, maxFood, fertile;
		int ix, iy;
		fertile = getFloat("FieldsInit.fertilePercentage");
		minFood = getFloat("FieldsInit.minFood");
		maxFood = getFloat("FieldsInit.maxFood");
		fields = new Field[getInt("World.fieldsX")][getInt("World.fieldsY")];
		for (iy=0; iy<World.getInt("World.fieldsY"); iy++){
			for (ix=0; ix<World.getInt("World.fieldsX"); ix++){
				if (fertile > 0 && random.nextFloat() < fertile){
					fields[ix][iy] = new Field(random.nextFloat()*(maxFood-minFood)+minFood, true);
				}
				else{
					fields[ix][iy] = new Field();
				}
			}
		}
	}

	private void agentsInit(){
		int width, height, fieldNum, agentNum;
		int xMin, yMin, ix, iy, x, y;
		float agentsPerField;
		int agentsPlaced;
		Agent a = null;
		width = getInt("AgentInit.width");
		height = getInt("AgentInit.height");
		if (width <= 0){width = getInt("World.fieldsX");}
		if (height <= 0){height = getInt("World.fieldsY");}
		xMin = (getInt("World.fieldsX") - width) / 2;
		yMin = (getInt("World.fieldsY") - height) / 2;
		fieldNum = width * height;
		agentNum = getInt("AgentInit.number");
		if (agentNum > fieldNum){agentNum = fieldNum;}
		agentsPerField = (float)agentNum / (float)fieldNum;
		agentsPlaced = 0;
		if (agentsPerField > 0.5){
			//alle Felder durchgehen und Agenten verteilen
			do{ //solange bis alle plaziert sind sind
				for (ix=xMin; ix<xMin+width; ix++){
					for (iy=yMin; iy<yMin+height; iy++){
						if (random.nextFloat() < agentsPerField 
								&& agentsPlaced < agentNum
								&& !getField(ix, iy).hasAgent()){
							a = new Agent(agentTreeRoot);
							Agent.agentNum++;
							a.setName("Agent" + Agent.agentNum);
							a.setX(ix);
							a.setY(iy);
							getField(ix, iy).setAgent(a);
							a.setDefaultParameters();
							a.setFood(getFloat("AgentInit.food"));
							addAgent(a);
							AgentTree.newAgent(agentTreeRoot);
							agentsPlaced++;
						}
					}
				}
			}
			while (agentsPlaced < agentNum);
		}
		else{
			//alle Agenten durchgehen und auf die Felder verteilen
			for (int i=1; i<=agentNum; i++){
				do {
					ix = xMin + (int)(width * random.nextFloat());
					iy = yMin + (int)(height * random.nextFloat());
				}
				while (fields[ix][iy].hasAgent());
				a = new Agent(agentTreeRoot);
				Agent.agentNum++;
				a.setName("Agent" + Agent.agentNum);
				a.setX(ix);
				a.setY(iy);
				fields[ix][iy].setAgent(a);
				a.setDefaultParameters();
				a.setFood(getFloat("AgentInit.food"));
				addAgent(a);
				AgentTree.newAgent(agentTreeRoot);
			}
		}
	}
	/**
	 * Initialisiert die props mit Defaultwerten.
	 */
	private void setProps(String fileName){
		Properties defProps = new Properties();
		FileInputStream inStream;
		
		defProps.setProperty("WorldView.width",  "400");
		defProps.setProperty("WorldView.height", "300");
		defProps.setProperty("World.fieldsX",           "40");
		defProps.setProperty("World.fieldsY",           "30");
	
		props = new Properties(defProps);
		try {
			inStream = new FileInputStream(fileName);
			props.load(inStream);
			inStream.close();
		}
		catch (Exception e){
			System.out.print("Es gab Probleme beim Laden ");
			System.out.println("der Properties aus der Datei: " + fileName);
			System.out.println("in World.setProps " + e);
			System.out.println("Die Default-Properties werden verwendet!");
		}
	}
	
	/**
	 * Die erste Mutationsfunktion, die eine starke Tendenz zur Mitte der beiden 
	 * biologischen Grenzen hat.
	**/
	public static float mutate(float min, float max, float value, float width){
		if (width == 0f) {
			return(value);
		}
		else if (width < 1f) {
			//min und max neu berechnen
			min = value - (value-min)*width;
			max = value + (max-value)*width;
		}
		float h = 2 / (max-min);
		float D_1 = h * min*min / (2*(value-min));
		float E_1 = h * min / (value-min);
		float F_1 = h / (2*(value-min));
		float D_2 = h  *(min*max-min*value+max*value)/(2*(value-max));
		float E_2 = h * max/(value-max);
		float F_2 = h / (2*(value-max));
		float x =(value-min)/(max-min);
		
		//float r = (float)Math.random();
		float r = World.random.nextFloat();
		if (r < x){
			return((float)(E_1+Math.sqrt(E_1*E_1-4*F_1*(D_1-r)))/(2*F_1));
		}
		else{
			return((float)(E_2+Math.sqrt(E_2*E_2-4*F_2*(D_2-r)))/(2*F_2));
		}
	}
	
	public static float mutate2(float min, float max, float value, float width){
		float min2, max2, width2;
		if (width == 0f) {
			return(value);
		}
		else if (width < 1f) {
			//min und max neu berechnen
			width2 = width * (max - min);
			min2 = value - width2 / 2f;
			max2 = value + width2 / 2f;
			if (min2 < min){
				min2 = min;
				max2 = min2 + width2;
			}
			if (max2 > max){
				max2 = max;
				min2 = max2 - width2;
			}
			min = min2;
			max = max2;
		}
		float h = 2 / (max-min);
		float D_1 = h * min*min / (2*(value-min));
		float E_1 = h * min / (value-min);
		float F_1 = h / (2*(value-min));
		float D_2 = h  *(min*max-min*value+max*value)/(2*(value-max));
		float E_2 = h * max/(value-max);
		float F_2 = h / (2*(value-max));
		float x =(value-min)/(max-min);
		
		float r = World.random.nextFloat();
		if (r < x){
			return((float)(E_1+Math.sqrt(E_1*E_1-4*F_1*(D_1-r)))/(2*F_1));
		}
		else{
			return((float)(E_2+Math.sqrt(E_2*E_2-4*F_2*(D_2-r)))/(2*F_2));
		}
	}
		
	public static void initAudioClips(){
		String fs = System.getProperty("file.separator");
		try{
			birthSound = Applet.newAudioClip(new URL("file:sound" + fs + "birth.wav"));
			deathSound = Applet.newAudioClip(new URL("file:sound" + fs + "death.wav"));
		}
		catch(Exception e){
			System.out.println("in World.initAudioClips: Problems loading Sound files.");
		}
	}
		
		
	public static Vector getAgentsClone(){
		return((Vector)agents.clone());
	}

	public static void initAgentTreeRoot(){
		agentTreeRoot = new Agent((Agent)null);
		agentTreeRoot.setName("Agent");
		agentTreeRoot.setDefaultParameters(); //auch wenns vielleicht nicht so dringend ist
	}

	public static TreeNode getAgentTreeRoot(){
		return(agentTreeRoot);
	}

	public static Agent getAgent(String name){
		for (int i=0; i<agents.size(); i++){
			if (((Agent)agents.elementAt(i)).getName().toString().equals(name)){
				return((Agent)agents.elementAt(i));
			}
		}
		System.out.println("in World.getAgent() null" + name);
		return(null);
	}

	public static void addAgent(Agent a){
		agents.add(a);
	}
	public static void removeAgent(Agent a){
		agents.remove(a);
	}

	/**
	 * Setzt den editMode.
	 * @param em true wenn editiert werden kann, false wenn eine Simulation luft.
	**/
	public static void setEditMode(boolean em){
		editMode = em;
		((MyMenuBar)(worldView.getMenuBar())).setEditEnabled(em);
		worldView.simControl.setEditMode(em);
	}
	
	/**
	 * Sagt ob gerade editiert werden kann oder nicht. 
	 * @return true falls die Welt editiert werden kann (Simulations-Pause), false falls
	 * kein Editieren mglich (Simualtion luft)
	**/
	public static boolean getEditMode(){
		return(editMode);
	}
	
	/**
	 * Setzt den FeldEditierCursor auf die neue Position. Kmmert sich nicht um eine
	 * Bildschirmaktualisierung.
	 * @param p die neue Cursor-Position.
	**/
	public static void setCursorPosition(Point p){
		cursorPosition = p;
		
	}
	
	/**
	 * Liefert die aktuelle Position des EditierCursors.
	 * @return die aktuelle Position.
	**/
	public static Point getCursorPosition(){
		return(cursorPosition);
	}
	
	public static Point torusDirection(int x0, int y0, int x1, int y1){
		int dx = x1 - x0;
		int dy = y1 - y0;
		if (dx < -World.getInt("World.fieldsX")/2) 
			dx += World.getInt("World.fieldsX");
		if (dx > World.getInt("World.fieldsX")/2) 
			dx -= World.getInt("World.fieldsX");
		if (dy < -World.getInt("World.fieldsY")/2) 
			dy += World.getInt("World.fieldsY");
		if (dy > World.getInt("World.fieldsY")/2) 
			dy -= World.getInt("World.fieldsY");
		
		if (dx > 0) dx =  1;
		if (dx < 0) dx = -1;
		if (dy > 0) dy =  1;
		if (dy < 0) dy = -1;
		
		return(new Point(dx, dy));
	}
	public static int distance(int x0, int y0, int x1, int y1){
		int dx = Math.abs(x1 - x0);
		int dy = Math.abs(y1 - y0);
		if (dx > World.getInt("World.fieldsX")/2) 
			dx -= World.getInt("World.fieldsX");
		if (dy > World.getInt("World.fieldsY")/2) 
			dy -= World.getInt("World.fieldsY");
		
		return(Math.max(dx, dy));
	}
	public static void setStatusLine(StringBuffer text){
		setStatusLine(text.toString());
	}
	public static void setStatusLine(String text){
		worldView.statusLine.setText(" " + text);
	}

	public static String getProp(String key){
		return(props.getProperty(key));
	}
	public static void setProp(String key, String value){
		props.setProperty(key, value);
	}
	public static void setInt(String key, int value){
		props.setProperty(key, (new Integer(value)).toString());
	}
	
	/**
	 * Liefert die angegebene Property als int zurck. Wenn was schiefluft gibts eine: 
	 * NumberFormatException, die das Programm vorzeitig beendet.
	 * @param s Der Name der gewnschten Property
	 * @return der aktuelle Wert als int. 
	**/
	public static int getInt(String s){
		return(Integer.parseInt(props.getProperty(s), 10));
	}
	public static float getFloat(String s){
		return(Float.parseFloat(props.getProperty(s)));
	}
	public static boolean getBoolean(String s){
		return(Boolean.valueOf(props.getProperty(s)).booleanValue());
	}
	
	
	/**
	 * Liefert zu den Koordinaten das passende Field.
	 * @param x die x-Koordinate des gewnschten Feldes.
	 * @param x die x-Koordinate des gewnschten Feldes.
	 * @return das Field mit den angegebenen Koordinaten.
	**/
	public static Field getField(int x, int y){
		if (x >= getInt("World.fieldsX"))
			x = x - getInt("World.fieldsX");
		if (x < 0)
			x = x + getInt("World.fieldsX");
		if (y >= getInt("World.fieldsY"))
			y = y - getInt("World.fieldsY");
		if (y < 0)
			y = y + getInt("World.fieldsY");
		return(fields[x][y]);
	}

	public static Vector getCoordinatesInDistance(int x, int y, int d){
			int i, dx, dy;
			Vector v = new Vector();
			for (i=-d; i<=d; i++){
				// obere Reihe
				dx = i;
				dy = -d;
				v.add(new Point(correctX(x + dx), correctY(y + dy)));
				
				// untere Reihe
				dx = i;
				dy = d;
				v.add(new Point(correctX(x + dx), correctY(y + dy)));
			}
			for (i=1-d; i<d; i++){
				// linke Spalte
				dx = -d;
				dy = i;
				v.add(new Point(correctX(x + dx), correctY(y + dy)));
				
				// rechte Spalte
				dx = d;
				dy = i;
				v.add(new Point(correctX(x + dx), correctY(y + dy)));
			}
			return(v);
		}

	public static Vector getRandomizedCoordinatesInDistance(int x, int y, int d){
		Vector v = getCoordinatesInDistance(x, y, d);
		Collections.shuffle(v, random);
		return(v);
	}
		
	public static int correctX(int x){
		while (x >= getInt("World.fieldsX")){
			x = x - getInt("World.fieldsX");
		}
		while (x < 0){
			x = x + getInt("World.fieldsX");
		}
		return(x);
	}
	public static int correctY(int y){
		while (y >= getInt("World.fieldsY")){
			y = y - getInt("World.fieldsY");
		}
		while (y < 0){
			y = y + getInt("World.fieldsY");
		}
		return(y);
	}
	
	/**
	 * 
	**/
	public static Point getFreeField(int x, int y){
		if (!fields[x][y].hasLivingAgent()){
			return(new Point(x, y));
		}
		Vector coords;
		int range = 1;
		while (range < World.getInt("World.fieldsX")/2 || range < World.getInt("World.fieldsY")/2){
			coords = World.getRandomizedCoordinatesInDistance(x, y, range);
			for (int i=0; i<coords.size(); i++) {
				Point p = (Point)(coords.elementAt(i));
				if (!fields[p.x][p.y].hasLivingAgent()){
					return(p);
				}
			}
			range++;
		}
		return(null);	//wenn alle Felder belegt sind
	}
	
	public static Polygon getCoordinatorPolygon(int x, int y, int width, int height){
		Polygon triangle = new Polygon();
		triangle.addPoint((int)(x+0.1*width), (int)(y+0.8*height));
		triangle.addPoint((int)(x+0.9*width), (int)(y+0.8*height));
		triangle.addPoint((int)(x+0.5*width), (int)(y+0.1*height));
		return(triangle);
	}

	public static Polygon getSubordinatePolygon(int x, int y, int width, int height){
		Polygon triangle = new Polygon();
		triangle.addPoint((int)(x+0.1*width), (int)(y+0.2*height));
		triangle.addPoint((int)(x+0.5*width), (int)(y+0.9*height));
		triangle.addPoint((int)(x+0.9*width), (int)(y+0.2*height));
		return(triangle);
	}
	/**
	 * Druckt die aktuellen Properties auf die Standardausgabe.
	**/
	public static void printProps(){
		props.list(new PrintStream(System.out));
	}

	/**
	 * Erweitert das standard write um das Speichern der Klassen-Variablen
	 * @param aus formalen Grnden...
	**/
 	private void writeObject(ObjectOutputStream objOut) throws IOException{
 		objOut.writeObject(props); 
 		objOut.writeObject(agents);
 		objOut.writeObject(fields);
 		objOut.writeInt(round);
  	objOut.writeObject(agentTreeRoot);
  	objOut.writeObject(random);
 	}
	/**
	 * Erweitert das standard read um das Laden der Klassen-Variablen
	 * @param aus formalen Grnden...	 
	**/
 	private void readObject(ObjectInputStream objIn) throws IOException, ClassNotFoundException{
 		props = (Properties)objIn.readObject(); 
 		agents = (Vector)objIn.readObject();

 		fields = (Field[][])objIn.readObject();
 		round = objIn.readInt();
 		agentTreeRoot = (Agent)objIn.readObject();
 		random = (Random)objIn.readObject();
 	}	
}