import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;

/**
 * Zeigt den aktuellen Zustand der Welt an.
**/
public class WorldView extends Frame implements WindowListener, ActionListener, ComponentListener{

	private World world;
	public TorusView torusView;
	public SimControl simControl;
	private Scheduler scheduler;
	public Label statusLine;
	
	/**
	 * Erschafft den Viewer zur Welt.
	 * @param w die anzuzeigende Welt
	**/
	public WorldView(World w){
		setTitle("World");
		setMenuBar(new MyMenuBar());
		world = w;
		int width = World.getInt("WorldView.width");
		int height = World.getInt("WorldView.height");
		simControl = new SimControl();
		torusView = new TorusView();
		statusLine = new Label(" ");
		this.setLayout(new BorderLayout());
		this.add(simControl, BorderLayout.NORTH);
		simControl.addListener((ActionListener)this);
		this.add(torusView,BorderLayout.CENTER);
		this.add(statusLine, BorderLayout.SOUTH);
		this.setSize(width, height);		
		this.addWindowListener(this);
		this.addComponentListener(this);
	}
	
	public void setScheduler(Scheduler s){
		scheduler = s;
	}
	public TorusView getTorusView(){
		return(torusView);
	}
	
	/**
	 * Speichert die aktuelle Welt.
	**/
	public void saveWorld() {
		try{
			JFileChooser c = new JFileChooser(new File("."));
			int returnVal = c.showSaveDialog(this); 
			if(returnVal == JFileChooser.APPROVE_OPTION) { 
				ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(c.getSelectedFile()));
				oos.writeObject(world);
				oos.close();
			}
		}
		catch (Exception e){
			System.out.println("in WorldView.saveWorld: " + e);
		}
	}

	public void saveWorld(String filename) {
		try{
			String pathAndName = "save";
			pathAndName = pathAndName + System.getProperty("file.separator");
			pathAndName = pathAndName + filename;
			ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(pathAndName));
				oos.writeObject(world);
				oos.close();
		}
		catch (Exception e){
			System.out.println("in WorldView.saveWorld: " + e);
		}
	}

	public void loadWorld() {
		try{
			JFileChooser c = new JFileChooser(new File("."));
			int returnVal = c.showOpenDialog(this); 
			if(returnVal == JFileChooser.APPROVE_OPTION) { 
				ObjectInputStream ois = new ObjectInputStream(new FileInputStream(c.getSelectedFile()));
				world = (World)ois.readObject();
				ois.close();
				this.setVisible(false);
				this.setVisible(true);
				updateRound();
			}
		}
		catch (Exception e){
			System.out.println("in WorldView.loadWorld: " + e);
		}
	}
	
	public void loadWorld(String filename) {
		try{
			ObjectInputStream ois = new ObjectInputStream(new FileInputStream(filename));
			world = (World)ois.readObject();
			ois.close();
			this.setVisible(false);
			this.setVisible(true);
		}
		catch (Exception e){
			System.out.println("in WorldView.loadWorld(name): " + e);
		}
		if (World.runFile){
			World.setEditMode(false);
			((MyMenuBar)getMenuBar()).setEditEnabled(false);			
			torusView.hideCursor();
			Scheduler.run = true;
		}
	}
	public void runSim(){
		if (World.runFile){
			World.setEditMode(false);
			((MyMenuBar)getMenuBar()).setEditEnabled(false);			
			torusView.hideCursor();
			Scheduler.run = true;
		}
	}
	public void startRecord(){
		try{
			JFileChooser c = new JFileChooser(new File("."));
			int returnVal = c.showSaveDialog(this); 
			if(returnVal == JFileChooser.APPROVE_OPTION) { 
				ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(c.getSelectedFile()));
				World.props.setProperty("VCR.fileVersion", "0.1");
				World.props.setProperty("VCR.startRound", new Integer(World.round).toString());
				out.writeObject(World.props);
				Vector initVector = new Vector();
				for (int j=0; j<World.getInt("World.fieldsY"); j++){
					for (int i=0; i<World.getInt("World.fieldsX"); i++){
						torusView.paintField(i, j, initVector);
					}
				}
				out.writeObject(initVector);
				scheduler.setRecordStream(out);
				scheduler.setRecordMode(true);
			}
		}
		catch (Exception e){
			System.out.println("in WorldView.startRecord: " + e);
		}	
	}
	
	
	public void editAgent(){
		Point p = World.getCursorPosition();
		if (World.getField(p.x, p.y).hasLivingAgent()){

			Agent a = World.getField(p.x, p.y).getAgent();
			a.setDeathDay(World.round);
			a.removeMemoryMonitor();
			a.removeAgentMonitor();
			a.setFood(0);
			scheduler.removeAgent(a);
			World.getField(p.x, p.y).setAgent(null);
			torusView.paintField(p.x, p.y, null);
		}
		else{

			Agent a = new Agent((Agent)(World.getAgentTreeRoot()));
			Agent.agentNum++;
			a.setName("Agent" + Agent.agentNum);
			a.setX(p.x);
			a.setY(p.y);
			World.getField(p.x, p.y).setAgent(a);
			a.setFood(World.getFloat("AgentInit.food"));
			a.setDefaultParameters();
			World.addAgent(a);
			AgentTree.newAgent((Agent)World.getAgentTreeRoot());
			torusView.paintField(p.x, p.y, null);
		}
	}
	
	public void editFood(){
		Point p = World.getCursorPosition();				
		float oldFood = World.getField(p.x, p.y).getFood();
		String inputValue = JOptionPane.showInputDialog(this,	"Old food value = " + oldFood + ". Enter new value: ");
		if (inputValue != null && !(inputValue.equals(""))){
			float newFood = (new Float(inputValue)).floatValue();
			World.getField(p.x, p.y).setFood(newFood);
			torusView.paintField(p.x, p.y, null);
		}
	}
	public void editFood100(){
		Point p = World.getCursorPosition();				
		World.getField(p.x, p.y).setFood(100);
		torusView.paintField(p.x, p.y, null);
	}
	public void agentsMenuSelected(String name){
		Agent a =	World.getAgent(name);
		if (a == null){
			System.out.println("in WorldView.agentsMenuSelected(" + name + ") agent not found!");
		}
		else{
			torusView.hideCursor();
			World.setCursorPosition(new Point(a.getX(), a.getY()));
			torusView.showCursor();
		}
	}
	
	public void setMemoryMonitor(){
		Point p = World.getCursorPosition();				
		if (World.getField(p.x, p.y).hasAgent())
			World.getField(p.x, p.y).getAgent().setMemoryMonitor();
	}
	
	public void setAgentMonitor(){
		Point p = World.getCursorPosition();				
		if (World.getField(p.x, p.y).hasAgent())
			World.getField(p.x, p.y).getAgent().setAgentMonitor();
	}
	
	public void updateRound(){
		simControl.updateRound(World.round);
	}

	public void editProps(){
		PropsEdit pe = new PropsEdit();
		pe.show();
	}

	public void printProps(){
		World.printProps();
	}

	public void slowDown(){
		int oldValue = World.getInt("World.slowDown");
		String inputValue = JOptionPane.showInputDialog(this,	"Actual pause beetween steps: " + oldValue +" ms. Enter new value: ");
		if (inputValue != null && !(inputValue.equals(""))){
			int newValue = (new Integer(inputValue)).intValue();
			if (newValue < 0) newValue = 0;
			World.setInt("World.slowDown", newValue);
		}
	}
	
	public void paintFoodOn(){
		torusView.paintFoodOn();
	}
	
	public void paintFoodOff(){
		torusView.paintFoodOff();	
	}
	public void showAgentTree(){
		AgentTree at = new AgentTree();
	}
	public void showHelp(){
		HelpView vw = new HelpView();
	}
	public void paintField(int x, int y){
		torusView.paintField(x, y, null);
	}
	
	/**
	 * Beendet das Programm, wenn das Hauptfenster geschlossen wird.
	**/
	public void windowClosing(WindowEvent e){
		if (World.sound) {
			try{
				Thread.sleep(2300);
			}catch(Exception x){}
		}
		System.exit(0);
	}
	public void windowOpened(WindowEvent e){}
	public void windowClosed(WindowEvent e){}
	public void windowIconified(WindowEvent e){}
	public void windowDeiconified(WindowEvent e){}
	public void windowActivated(WindowEvent e){}
	public void windowDeactivated(WindowEvent e){}
	
	public void actionPerformed(ActionEvent e){
		if (e.getActionCommand().equals("stop")){
			Scheduler.run = false;
			//World.setEditMode(true);
			//((MyMenuBar)getMenuBar()).setEditEnabled(true);
			torusView.showCursor();
		//	torusView.repaint();
		} 
		else if (e.getActionCommand().equals("step")){
			World.setEditMode(false);
			((MyMenuBar)getMenuBar()).setEditEnabled(false);						
			//torusView.repaint();			
			torusView.hideCursor();
			scheduler.step();
			World.setEditMode(true);
			torusView.showCursor();
			((MyMenuBar)getMenuBar()).setEditEnabled(true);
		//	torusView.repaint();			
		}
		else if (e.getActionCommand().equals("go")){
			World.setEditMode(false);
			((MyMenuBar)getMenuBar()).setEditEnabled(false);			
			torusView.hideCursor();
			Scheduler.run = true;
		}
		else {
			System.out.println("in WorldView.actionPerformed: unknownActionCommand: " + e);
		}
	}
	public void componentHidden(ComponentEvent e){}
	public void componentMoved(ComponentEvent e){}
	public void componentResized(ComponentEvent e){}
	public void componentShown(ComponentEvent e){}
}